 /* \file  bl_platform.c
 *
 * \brief Initializes AM335x Device Peripherals.
 *
 */

/*
* Copyright (C) 2010-2015 Texas Instruments Incorporated - http://www.ti.com/
*
*  Redistribution and use in source and binary forms, with or without
*  modification, are permitted provided that the following conditions
*  are met:
*
*    Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*
*    Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the
*    distribution.
*
*    Neither the name of Texas Instruments Incorporated nor the names of
*    its contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
*  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
*  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
*  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#include "hw_types.h"
#include "hw_cm_cefuse.h"
#include "hw_cm_device.h"
#include "hw_cm_dpll.h"
#include "hw_cm_gfx.h"
#include "hw_cm_mpu.h"
#include "hw_cm_per.h"
#include "hw_cm_rtc.h"
#include "hw_cm_wkup.h"
#include "hw_control_AM335x.h"
#include "hw_emif4d.h"
#include "bl.h"
#include "gpmc.h"
#include "hw_gpmc.h"
#include "bl_copy.h"
#include "bl_platform.h"
//#include "uartStdio.h"
#include "watchdog.h"
#include "hsi2c.h"
#include "gpio_v2.h"
#include "board.h"
#include "device.h"
#include "string.h"
#include "pruss.h"
#ifdef evmAM335x
    #include "hw_tps65910.h"
    #ifdef HW_ICE_V2
        #include "pin_mux.h"
    #endif
#elif  (defined beaglebone)
    #include "hw_tps65217.h"
#elif  (defined evmskAM335x)
    #include "hw_tps65910.h"
    #include "pin_mux.h"
#endif

#if defined(SPI)
    #include "bl_spi.h"
#elif defined(NAND)
    #include "bl_nand.h"
    #include "nandlib.h"
    #include "nand_gpmc.h"
#endif
#if defined(MMCSD)
    #include "bl_mmcsd.h"
#endif


extern void bl_UARTPuts(char* buffer);
/******************************************************************************
**                     Internal Macro Definitions
*******************************************************************************/
#define INTVECMAX                          (9)
#define BIT(n)                             (1 << (n))
#define PAD_CTRL_PULLUDDISABLE             (BIT(3))
#define PAD_CTRL_PULLUPSEL                 (BIT(4))
#define PAD_CTRL_RXACTIVE                  (BIT(5))
#define PAD_CTRL_SLOWSLEW                  (BIT(6))
#define PAD_CTRL_MUXMODE(n)                ((n))

/*	I2C instance	*/
#define  I2C_0				   (0x0u)
/*	System clock fed to I2C module - 48Mhz	*/
#define  I2C_SYSTEM_CLOCK		   (48000000u)
/*	Internal clock used by I2C module - 12Mhz	*/
#define  I2C_INTERNAL_CLOCK		   (12000000u)
/*	I2C bus speed or frequency - 100Khz	*/
#define	 I2C_OUTPUT_CLOCK		   (100000u)
/*	I2C interrupt flags to clear	*/
#define  I2C_INTERRUPT_FLAG_TO_CLR         (0x7FF)
#define  PMIC_SR_I2C_SLAVE_ADDR            (0x12)
#define SMPS_DRIVE_SCLSR_EN1	           (0x0u)
#define SMPS_DRIVE_SDASR_EN2	           (0x1u)

/* TODO : These are not there in the control module header file */
#define DDR_PHY_CTRL_REGS                  (SOC_CONTROL_REGS + 0x2000)
#define CMD0_SLAVE_RATIO_0                 (DDR_PHY_CTRL_REGS + 0x1C)
#define CMD0_SLAVE_FORCE_0                 (DDR_PHY_CTRL_REGS + 0x20)
#define CMD0_SLAVE_DELAY_0                 (DDR_PHY_CTRL_REGS + 0x24)
#define CMD0_LOCK_DIFF_0                   (DDR_PHY_CTRL_REGS + 0x28)
#define CMD0_INVERT_CLKOUT_0               (DDR_PHY_CTRL_REGS + 0x2C)
#define CMD1_SLAVE_RATIO_0                 (DDR_PHY_CTRL_REGS + 0x50)
#define CMD1_SLAVE_FORCE_0                 (DDR_PHY_CTRL_REGS + 0x54)
#define CMD1_SLAVE_DELAY_0                 (DDR_PHY_CTRL_REGS + 0x58)
#define CMD1_LOCK_DIFF_0                   (DDR_PHY_CTRL_REGS + 0x5C)
#define CMD1_INVERT_CLKOUT_0               (DDR_PHY_CTRL_REGS + 0x60)
#define CMD2_SLAVE_RATIO_0                 (DDR_PHY_CTRL_REGS + 0x84)
#define CMD2_SLAVE_FORCE_0                 (DDR_PHY_CTRL_REGS + 0x88)
#define CMD2_SLAVE_DELAY_0                 (DDR_PHY_CTRL_REGS + 0x8C)
#define CMD2_LOCK_DIFF_0                   (DDR_PHY_CTRL_REGS + 0x90)
#define CMD2_INVERT_CLKOUT_0               (DDR_PHY_CTRL_REGS + 0x94)
#define DATA0_RD_DQS_SLAVE_RATIO_0         (DDR_PHY_CTRL_REGS + 0xC8)
#define DATA0_RD_DQS_SLAVE_RATIO_1         (DDR_PHY_CTRL_REGS + 0xCC)
#define DATA0_WR_DQS_SLAVE_RATIO_0         (DDR_PHY_CTRL_REGS + 0xDC)
#define DATA0_WR_DQS_SLAVE_RATIO_1         (DDR_PHY_CTRL_REGS + 0xE0)
#define DATA0_WRLVL_INIT_RATIO_0           (DDR_PHY_CTRL_REGS + 0xF0)
#define DATA0_WRLVL_INIT_RATIO_1           (DDR_PHY_CTRL_REGS + 0xF4)
#define DATA0_GATELVL_INIT_RATIO_0         (DDR_PHY_CTRL_REGS + 0xFC)
#define DATA0_GATELVL_INIT_RATIO_1         (DDR_PHY_CTRL_REGS + 0x100)
#define DATA0_FIFO_WE_SLAVE_RATIO_0        (DDR_PHY_CTRL_REGS + 0x108)
#define DATA0_FIFO_WE_SLAVE_RATIO_1        (DDR_PHY_CTRL_REGS + 0x10C)
#define DATA0_WR_DATA_SLAVE_RATIO_0        (DDR_PHY_CTRL_REGS + 0x120)
#define DATA0_WR_DATA_SLAVE_RATIO_1        (DDR_PHY_CTRL_REGS + 0x124)
#define DATA0_USE_RANK0_DELAYS_0           (DDR_PHY_CTRL_REGS + 0x134)
#define DATA0_LOCK_DIFF_0                  (DDR_PHY_CTRL_REGS + 0x138)
#define DATA1_RD_DQS_SLAVE_RATIO_0         (DDR_PHY_CTRL_REGS + 0x16c)
#define DATA1_RD_DQS_SLAVE_RATIO_1         (DDR_PHY_CTRL_REGS + 0x170)
#define DATA1_WR_DQS_SLAVE_RATIO_0         (DDR_PHY_CTRL_REGS + 0x180)
#define DATA1_WR_DQS_SLAVE_RATIO_1         (DDR_PHY_CTRL_REGS + 0x184)
#define DATA1_WRLVL_INIT_RATIO_0           (DDR_PHY_CTRL_REGS + 0x194)
#define DATA1_WRLVL_INIT_RATIO_1           (DDR_PHY_CTRL_REGS + 0x198)
#define DATA1_GATELVL_INIT_RATIO_0         (DDR_PHY_CTRL_REGS + 0x1a0)
#define DATA1_GATELVL_INIT_RATIO_1         (DDR_PHY_CTRL_REGS + 0x1a4)
#define DATA1_FIFO_WE_SLAVE_RATIO_0        (DDR_PHY_CTRL_REGS + 0x1ac)
#define DATA1_FIFO_WE_SLAVE_RATIO_1        (DDR_PHY_CTRL_REGS + 0x1b0)
#define DATA1_WR_DATA_SLAVE_RATIO_0        (DDR_PHY_CTRL_REGS + 0x1c4)
#define DATA1_WR_DATA_SLAVE_RATIO_1        (DDR_PHY_CTRL_REGS + 0x1c8)
#define DATA1_USE_RANK0_DELAYS_0           (DDR_PHY_CTRL_REGS + 0x1d8)
#define DATA1_LOCK_DIFF_0                  (DDR_PHY_CTRL_REGS + 0x1dc)


/* DDR3 init values */
#ifdef evmAM335x
#ifdef DDR3_400MHZ
#define DDR3_CMD0_SLAVE_RATIO_0            (0x80)
#define DDR3_CMD0_INVERT_CLKOUT_0          (0x0)
#define DDR3_CMD1_SLAVE_RATIO_0            (0x80)
#define DDR3_CMD1_INVERT_CLKOUT_0          (0x0)
#define DDR3_CMD2_SLAVE_RATIO_0            (0x80)
#define DDR3_CMD2_INVERT_CLKOUT_0          (0x0)

#define DDR3_DATA0_RD_DQS_SLAVE_RATIO_0    (0x3A)
#define DDR3_DATA0_FIFO_WE_SLAVE_RATIO_0   (0x96)
#define DDR3_DATA0_WR_DQS_SLAVE_RATIO_0    (0x3B)
#define DDR3_DATA0_WR_DATA_SLAVE_RATIO_0   (0x76)

#define DDR3_DATA0_RD_DQS_SLAVE_RATIO_1    (0x3A)
#define DDR3_DATA0_FIFO_WE_SLAVE_RATIO_1   (0x96)
#define DDR3_DATA0_WR_DQS_SLAVE_RATIO_1    (0x3B)
#define DDR3_DATA0_WR_DATA_SLAVE_RATIO_1   (0x76)

#define DDR3_CONTROL_DDR_CMD_IOCTRL_0      (0x18B)
#define DDR3_CONTROL_DDR_CMD_IOCTRL_1      (0x18B)
#define DDR3_CONTROL_DDR_CMD_IOCTRL_2      (0x18B)

#define DDR3_CONTROL_DDR_DATA_IOCTRL_0      (0x18B)
#define DDR3_CONTROL_DDR_DATA_IOCTRL_1      (0x18B)

#define DDR3_CONTROL_DDR_IO_CTRL           (0xefffffff)

#define DDR3_EMIF_DDR_PHY_CTRL_1           (0x07)
#define DDR3_EMIF_DDR_PHY_CTRL_1_DY_PWRDN         (0x00100000)
#define DDR3_EMIF_DDR_PHY_CTRL_1_SHDW      (0x07)
#define DDR3_EMIF_DDR_PHY_CTRL_1_SHDW_DY_PWRDN    (0x00100000)
#define DDR3_EMIF_DDR_PHY_CTRL_2           (0x07)

#define DDR3_EMIF_SDRAM_TIM_1              (0x0AAAD4DB)
#define DDR3_EMIF_SDRAM_TIM_1_SHDW         (0x0AAAD4DB)

#define DDR3_EMIF_SDRAM_TIM_2              (0x26437FDA)
#define DDR3_EMIF_SDRAM_TIM_2_SHDW         (0x26437FDA)

#define DDR3_EMIF_SDRAM_TIM_3              (0x501F83FF)
#define DDR3_EMIF_SDRAM_TIM_3_SHDM         (0x501F83FF)

#define DDR3_EMIF_SDRAM_REF_CTRL_VAL1      (0x00000C30)
#define DDR3_EMIF_SDRAM_REF_CTRL_SHDW_VAL1 (0x00000C30)

#define DDR3_EMIF_ZQ_CONFIG_VAL            (0x50074BE4)
#define DDR3_EMIF_SDRAM_CONFIG             (0x61C052B2)//termination = 1 (RZQ/4)
													   //dynamic ODT = 2 (RZQ/2)
                                                       //SDRAM drive = 0 (RZQ/6)
                                                       //CWL = 0 (CAS write latency = 5)
                                                       //CL=4 (CAS latency=6)
                                                       //ROWSIZE = 5 (14 row bits)
                                                       //PAGESIZE = 2 (10 column bits)
#else
#define DDR3_CMD0_SLAVE_RATIO_0            (0x80)
#define DDR3_CMD0_INVERT_CLKOUT_0          (0x0)
#define DDR3_CMD1_SLAVE_RATIO_0            (0x80)
#define DDR3_CMD1_INVERT_CLKOUT_0          (0x0)
#define DDR3_CMD2_SLAVE_RATIO_0            (0x80)
#define DDR3_CMD2_INVERT_CLKOUT_0          (0x0)

#define DDR3_DATA0_RD_DQS_SLAVE_RATIO_0    (0x3B)
#define DDR3_DATA0_WR_DQS_SLAVE_RATIO_0    (0x3C)
#define DDR3_DATA0_FIFO_WE_SLAVE_RATIO_0   (0xA5)
#define DDR3_DATA0_WR_DATA_SLAVE_RATIO_0   (0x74)

#define DDR3_DATA0_RD_DQS_SLAVE_RATIO_1    (0x3B)
#define DDR3_DATA0_WR_DQS_SLAVE_RATIO_1    (0x3C)
#define DDR3_DATA0_FIFO_WE_SLAVE_RATIO_1   (0xA5)
#define DDR3_DATA0_WR_DATA_SLAVE_RATIO_1   (0x74)

#define DDR3_CONTROL_DDR_CMD_IOCTRL_0      (0x18B)
#define DDR3_CONTROL_DDR_CMD_IOCTRL_1      (0x18B)
#define DDR3_CONTROL_DDR_CMD_IOCTRL_2      (0x18B)

#define DDR3_CONTROL_DDR_DATA_IOCTRL_0      (0x18B)
#define DDR3_CONTROL_DDR_DATA_IOCTRL_1      (0x18B)

#define DDR3_CONTROL_DDR_IO_CTRL           (0xefffffff)

#define DDR3_EMIF_DDR_PHY_CTRL_1           (0x06)
#define DDR3_EMIF_DDR_PHY_CTRL_1_DY_PWRDN         (0x00100000)
#define DDR3_EMIF_DDR_PHY_CTRL_1_SHDW      (0x06)
#define DDR3_EMIF_DDR_PHY_CTRL_1_SHDW_DY_PWRDN    (0x00100000)
#define DDR3_EMIF_DDR_PHY_CTRL_2           (0x06)

#define DDR3_EMIF_SDRAM_TIM_1              (0x0888A39B)
#define DDR3_EMIF_SDRAM_TIM_1_SHDW         (0x0888A39B)

#define DDR3_EMIF_SDRAM_TIM_2              (0x26517FDA)
#define DDR3_EMIF_SDRAM_TIM_2_SHDW         (0x26517FDA)

#define DDR3_EMIF_SDRAM_TIM_3              (0x501F84EF)
#define DDR3_EMIF_SDRAM_TIM_3_SHDM         (0x501F84EF)

#define DDR3_EMIF_SDRAM_REF_CTRL_VAL1      (0x0000093B)
#define DDR3_EMIF_SDRAM_REF_CTRL_SHDW_VAL1 (0x0000093B)

#define DDR3_EMIF_ZQ_CONFIG_VAL            (0x50074BE4)

/*
** termination = 1 (RZQ/4)
** dynamic ODT = 2 (RZQ/2)
** SDRAM drive = 0 (RZQ/6)
** CWL = 0 (CAS write latency = 5)
** CL = 2 (CAS latency = 5)
** ROWSIZE = 7 (16 row bits)
** PAGESIZE = 2 (10 column bits)
*/
#define DDR3_EMIF_SDRAM_CONFIG             (0x61C04BB2)
#endif
#else

#define DDR3_CMD0_SLAVE_RATIO_0            (0x40)
#define DDR3_CMD0_INVERT_CLKOUT_0          (0x1)
#define DDR3_CMD1_SLAVE_RATIO_0            (0x40)
#define DDR3_CMD1_INVERT_CLKOUT_0          (0x1)
#define DDR3_CMD2_SLAVE_RATIO_0            (0x40)
#define DDR3_CMD2_INVERT_CLKOUT_0          (0x1)

#define DDR3_DATA0_RD_DQS_SLAVE_RATIO_0    (0x3B)
#define DDR3_DATA0_WR_DQS_SLAVE_RATIO_0    (0x85)
#define DDR3_DATA0_FIFO_WE_SLAVE_RATIO_0   (0x100)
#define DDR3_DATA0_WR_DATA_SLAVE_RATIO_0   (0xC1)

#define DDR3_DATA0_RD_DQS_SLAVE_RATIO_1    (0x3B)
#define DDR3_DATA0_WR_DQS_SLAVE_RATIO_1    (0x85)
#define DDR3_DATA0_FIFO_WE_SLAVE_RATIO_1   (0x100)
#define DDR3_DATA0_WR_DATA_SLAVE_RATIO_1   (0xC1)

#define DDR3_CONTROL_DDR_CMD_IOCTRL_0      (0x18B)
#define DDR3_CONTROL_DDR_CMD_IOCTRL_1      (0x18B)
#define DDR3_CONTROL_DDR_CMD_IOCTRL_2      (0x18B)

#define DDR3_CONTROL_DDR_DATA_IOCTRL_0      (0x18B)
#define DDR3_CONTROL_DDR_DATA_IOCTRL_1      (0x18B)

//#define DDR3_CONTROL_DDR_IO_CTRL           (0x0fffffff)
#define DDR3_CONTROL_DDR_IO_CTRL           (0xefffffff)

#define DDR3_EMIF_DDR_PHY_CTRL_1           (0x06)
#define DDR3_EMIF_DDR_PHY_CTRL_1_DY_PWRDN         (0x00100000)
#define DDR3_EMIF_DDR_PHY_CTRL_1_SHDW      (0x06)
#define DDR3_EMIF_DDR_PHY_CTRL_1_SHDW_DY_PWRDN    (0x00100000)
#define DDR3_EMIF_DDR_PHY_CTRL_2           (0x06)

#define DDR3_EMIF_SDRAM_TIM_1              (0x0888A39B)
#define DDR3_EMIF_SDRAM_TIM_1_SHDW         (0x0888A39B)

#define DDR3_EMIF_SDRAM_TIM_2              (0x26337FDA)
#define DDR3_EMIF_SDRAM_TIM_2_SHDW         (0x26337FDA)

#define DDR3_EMIF_SDRAM_TIM_3              (0x501F830F)
#define DDR3_EMIF_SDRAM_TIM_3_SHDM         (0x501F830F)

#define DDR3_EMIF_SDRAM_REF_CTRL_VAL1      (0x0000093B)
#define DDR3_EMIF_SDRAM_REF_CTRL_SHDW_VAL1 (0x0000093B)

#define DDR3_EMIF_ZQ_CONFIG_VAL            (0x50074BE4)
#define DDR3_EMIF_SDRAM_CONFIG             (0x61C04AB2)//termination = 1 (RZQ/4)
                                                       //dynamic ODT = 2 (RZQ/2)
                                                       //SDRAM drive = 0 (RZQ/6)
                                                       //CWL = 0 (CAS write latency = 5)
                                                       //CL = 2 (CAS latency = 5)
                                                       //ROWSIZE = 5 (14 row bits)
                                                       //PAGESIZE = 2 (10 column bits)
													   
#endif

/* DDR2 init values */

#define DDR2_CMD0_SLAVE_RATIO_0            (0x80)
#define DDR2_CMD0_SLAVE_FORCE_0            (0x0)
#define DDR2_CMD0_SLAVE_DELAY_0            (0x0)
#define DDR2_CMD0_LOCK_DIFF_0              (0x4)
#define DDR2_CMD0_INVERT_CLKOUT_0          (0x0)

#define DDR2_CMD1_SLAVE_RATIO_0            (0x80)
#define DDR2_CMD1_SLAVE_FORCE_0            (0x0)
#define DDR2_CMD1_SLAVE_DELAY_0            (0x0)
#define DDR2_CMD1_LOCK_DIFF_0              (0x4)
#define DDR2_CMD1_INVERT_CLKOUT_0          (0x0)

#define DDR2_CMD2_SLAVE_RATIO_0            (0x80)
#define DDR2_CMD2_SLAVE_FORCE_0            (0x0)
#define DDR2_CMD2_SLAVE_DELAY_0            (0x0)
#define DDR2_CMD2_LOCK_DIFF_0              (0x4)
#define DDR2_CMD2_INVERT_CLKOUT_0          (0x0)

#define DDR2_DATA0_RD_DQS_SLAVE_RATIO_0    (0x12)
#define DDR2_DATA0_WR_DQS_SLAVE_RATIO_0    (0x0)
#define DDR2_DATA0_FIFO_WE_SLAVE_RATIO_0   (0x80)
#define DDR2_DATA0_WR_DATA_SLAVE_RATIO_0   (0x40)

#define DDR2_DATA1_RD_DQS_SLAVE_RATIO_0    (0x12)
#define DDR2_DATA1_WR_DQS_SLAVE_RATIO_0    (0x0)
#define DDR2_DATA1_FIFO_WE_SLAVE_RATIO_0   (0x80)
#define DDR2_DATA1_WR_DATA_SLAVE_RATIO_0   (0x40)

#define DDR2_CONTROL_DDR_CMD_IOCTRL_0      (0x18B)
#define DDR2_CONTROL_DDR_CMD_IOCTRL_1      (0x18B)
#define DDR2_CONTROL_DDR_CMD_IOCTRL_2      (0x18B)
#define DDR2_CONTROL_DDR_DATA_IOCTRL_0     (0x18B)
#define DDR2_CONTROL_DDR_DATA_IOCTRL_1     (0x18B)

#define DDR2_CONTROL_DDR_IO_CTRL           (0x0fffffff)
#define DDR2_EMIF_DDR_PHY_CTRL_1           (0x05)
#define DDR2_EMIF_DDR_PHY_CTRL_1_DY_PWRDN         (0x00100000)
#define DDR2_EMIF_DDR_PHY_CTRL_1_SHDW      (0x05)
#define DDR2_EMIF_DDR_PHY_CTRL_1_SHDW_DY_PWRDN    (0x00100000)
#define DDR2_EMIF_DDR_PHY_CTRL_2           (0x05)

#define DDR2_EMIF_SDRAM_TIM_1		   (0x0666B3D6)
#define DDR2_EMIF_SDRAM_TIM_1_SHDW         (0x0666B3D6)
#define DDR2_EMIF_SDRAM_TIM_2              (0x143731DA)
#define DDR2_EMIF_SDRAM_TIM_2_SHDW	   (0x143731DA)
#define DDR2_EMIF_SDRAM_TIM_3              (0x00000347)
#define DDR2_EMIF_SDRAM_TIM_3_SHDM         (0x00000347)
#define DDR2_EMIF_SDRAM_CONFIG             (0x41805332)
#define DDR2_EMIF_SDRAM_REF_CTRL_VAL1	   (0x00004650)
#define DDR2_EMIF_SDRAM_REF_CTRL_SHDW_VAL1 (0x00004650)

#define DDR2_EMIF_SDRAM_REF_CTRL_VAL2	   (0x0000081a)
#define DDR2_EMIF_SDRAM_REF_CTRL_SHDW_VAL2 (0x0000081a)

#ifdef HW_ICE_V2
	#define GPIO_INSTANCE_PIN_NUMBER      (18)
#else
	#define GPIO_INSTANCE_PIN_NUMBER      (7)
#endif


#define CTRL_MODULE_BASE_ADDR 	0x44E10000
/* Function mode macros */
#define PINMUXMODE_0	0x00
#define PINMUXMODE_1	0x01
#define PINMUXMODE_2	0x02
#define PINMUXMODE_3	0x03
#define PINMUXMODE_4	0x04
#define PINMUXMODE_5	0x05
#define PINMUXMODE_6	0x06
#define PINMUXMODE_7	0x07


/* NOR timing related defines */
#define GPMC_BASE_ADDR			0x50000000
#define GPMC_BASE				0x50000000

#define CS_ON_TIME			0
#define CS_DEASSERT_WR		0x1E	// ToDo
#define CS_DEASSERT_RD		9

#define ADV_ASSERT			0
#define ADV_DEASSERT_RD		1
#define ADV_DEASSERT_WR		3		// ToDo

#define OE_ASSERT 			2
#define OE_DEASSERT			9
#define WE_ASSERT			0xD		// ToDo
#define WE_DEASSERT			0x1C	// ToDo

#define RD_ACCESS_TIM		8
#define RD_CYCLE_TIM		12
#define WR_CYCLE_TIM		0x1F	// ToDo

#define CYC2CYC_DELAY		0
#define WR_DATA_ON_ADMUX	0x07	//ToDo
#define	WR_ACCESS_TIM		0xD		//ToDo


#define NOR_BASE_ADDR			0x08
#define NOR_MASK				0x0F


/******************************************************************************
**                     Local function Declarations
*******************************************************************************/

#if defined (evmskAM335x) || defined (HW_ICE_V2)
static void DDRVTTEnable(void);
#endif
extern void SPIConfigure(void);
extern void I2C1ModuleClkConfig(void);
void GPMCPinMuxConfNor(void);

/******************************************************************************
**                     Global variable Definitions
*******************************************************************************/
char *deviceType = "AM335x";
volatile unsigned char dataFromSlave[2];
volatile unsigned char dataToSlave[3];
volatile unsigned int tCount;
volatile unsigned int rCount;
volatile unsigned int flag = 1;
volatile unsigned int numOfBytes;
volatile unsigned int oppMaxIdx;
volatile unsigned int deviceVersion;
volatile unsigned int freqMultDDR;

#define NUM_OPP_ENTRIES     10
/*
** OPP table for mpu multiplier and pmic voltage select.
** MPUPLL_N and MPUPLL_M2 are divider and post divider values.
*/
tOPPConfig oppTable[NUM_OPP_ENTRIES];

#if defined(NAND)
  static GPMCNANDTimingInfo_t nandTimingInfo;
#endif


/******************************************************************************
**                     Local variable Definitions
*******************************************************************************/

/******************************************************************************
**                     Function Definitions
*******************************************************************************/

/*
** Determine maximum OPP configuration of SoC.
*/
unsigned int BootMaxOppGet(void)
{
    unsigned int oppIdx;
    unsigned int oppSupport = SysConfigOppDataGet();

    if(DEVICE_VERSION_1_0 == deviceVersion)
    {
        oppIdx = EFUSE_OPPTB_720;
    }
    else if(DEVICE_VERSION_2_0 == deviceVersion)
    {
        oppIdx = EFUSE_OPPTB_800;
    }
    else if(DEVICE_VERSION_2_1 == deviceVersion)
    {
        if(!(oppSupport & EFUSE_OPPNT_1000_MASK))
        {
            oppIdx = EFUSE_OPPNT_1000;
        }
        else if(!(oppSupport & EFUSE_OPPTB_800_MASK))
        {
            oppIdx = EFUSE_OPPTB_800;
        }
        else if(!(oppSupport & EFUSE_OPP120_720_MASK))
        {
            oppIdx = EFUSE_OPP120_720;
        }
        else if(!(oppSupport & EFUSE_OPP100_600_MASK))
        {
            oppIdx = EFUSE_OPP100_600;
        }
        else if(!(oppSupport & EFUSE_OPP100_300_MASK))
        {
            oppIdx = EFUSE_OPP100_300;
        }
        else
        {
            oppIdx = EFUSE_OPP50_300;
        }
    }
    else
    {
        return OPP_NONE;
    }

    return oppIdx;
}

/* \brief This function initializes the CORE PLL 
 * 
 * \param none
 *
 * \return none
 *
 */
void CorePLLInit(void)
{
    volatile unsigned int regVal = 0;

    /* Enable the Core PLL */

    /* Put the PLL in bypass mode */
    regVal = HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_CLKMODE_DPLL_CORE) &
                ~CM_WKUP_CM_CLKMODE_DPLL_CORE_DPLL_EN;

    regVal |= CM_WKUP_CM_CLKMODE_DPLL_CORE_DPLL_EN_DPLL_MN_BYP_MODE;

    HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_CLKMODE_DPLL_CORE) = regVal;

    while(!(HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_IDLEST_DPLL_CORE) &
                      CM_WKUP_CM_IDLEST_DPLL_CORE_ST_MN_BYPASS));

    /* Set the multipler and divider values for the PLL */
    HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_CLKSEL_DPLL_CORE) =
        ((COREPLL_M << CM_WKUP_CM_CLKSEL_DPLL_CORE_DPLL_MULT_SHIFT) |
         (COREPLL_N << CM_WKUP_CM_CLKSEL_DPLL_CORE_DPLL_DIV_SHIFT));

    /* Configure the High speed dividers */
    /* Set M4 divider */    
    regVal = HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_DIV_M4_DPLL_CORE);
    regVal = regVal & ~CM_WKUP_CM_DIV_M4_DPLL_CORE_HSDIVIDER_CLKOUT1_DIV;
    regVal = regVal | (COREPLL_HSD_M4 << 
                CM_WKUP_CM_DIV_M4_DPLL_CORE_HSDIVIDER_CLKOUT1_DIV_SHIFT);
    HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_DIV_M4_DPLL_CORE) = regVal;
    
    /* Set M5 divider */    
    regVal = HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_DIV_M5_DPLL_CORE);
    regVal = regVal & ~CM_WKUP_CM_DIV_M5_DPLL_CORE_HSDIVIDER_CLKOUT2_DIV;
    regVal = regVal | (COREPLL_HSD_M5 << 
                CM_WKUP_CM_DIV_M5_DPLL_CORE_HSDIVIDER_CLKOUT2_DIV_SHIFT);
    HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_DIV_M5_DPLL_CORE) = regVal;        
        
    /* Set M6 divider */    
    regVal = HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_DIV_M6_DPLL_CORE);
    regVal = regVal & ~CM_WKUP_CM_DIV_M6_DPLL_CORE_HSDIVIDER_CLKOUT3_DIV;
    regVal = regVal | (COREPLL_HSD_M6 << 
                CM_WKUP_CM_DIV_M6_DPLL_CORE_HSDIVIDER_CLKOUT3_DIV_SHIFT);
    HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_DIV_M6_DPLL_CORE) = regVal;         

    /* Now LOCK the PLL by enabling it */
    regVal = HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_CLKMODE_DPLL_CORE) &
                ~CM_WKUP_CM_CLKMODE_DPLL_CORE_DPLL_EN;

    regVal |= CM_WKUP_CM_CLKMODE_DPLL_CORE_DPLL_EN;

    HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_CLKMODE_DPLL_CORE) = regVal;

    while(!(HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_IDLEST_DPLL_CORE) &
                        CM_WKUP_CM_IDLEST_DPLL_CORE_ST_DPLL_CLK));
}

/* \brief This function initializes the DISPLAY PLL
 * 
 * \param none
 *
 * \return none
 *
 */
void DisplayPLLInit(void)
{
    volatile unsigned int regVal = 0;

    /* Put the PLL in bypass mode */
    regVal = HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_CLKMODE_DPLL_DISP) &
                ~CM_WKUP_CM_CLKMODE_DPLL_DISP_DPLL_EN;

    regVal |= CM_WKUP_CM_CLKMODE_DPLL_DISP_DPLL_EN_DPLL_MN_BYP_MODE;

    HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_CLKMODE_DPLL_DISP) = regVal;

    while(!(HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_IDLEST_DPLL_DISP) &
                        CM_WKUP_CM_IDLEST_DPLL_DISP_ST_MN_BYPASS));

    HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_CLKSEL_DPLL_DISP) &=
                           ~(CM_WKUP_CM_CLKSEL_DPLL_DISP_DPLL_DIV |
                             CM_WKUP_CM_CLKSEL_DPLL_DISP_DPLL_MULT);

    /* Set the multipler and divider values for the PLL */
    HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_CLKSEL_DPLL_DISP) |=
        ((DISPLL_M << CM_WKUP_CM_CLKSEL_DPLL_DISP_DPLL_MULT_SHIFT) |
         (DISPLL_N << CM_WKUP_CM_CLKSEL_DPLL_DISP_DPLL_DIV_SHIFT));

    regVal = HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_DIV_M2_DPLL_DISP);
    regVal = regVal & ~CM_WKUP_CM_DIV_M2_DPLL_DISP_DPLL_CLKOUT_DIV;
    regVal = regVal | DISPLL_M2;

    /* Set the CLKOUT2 divider */
    HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_DIV_M2_DPLL_DISP) = regVal;

    /* Now LOCK the PLL by enabling it */
    regVal = HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_CLKMODE_DPLL_DISP) &
                ~CM_WKUP_CM_CLKMODE_DPLL_DISP_DPLL_EN;

    regVal |= CM_WKUP_CM_CLKMODE_DPLL_DISP_DPLL_EN;

    HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_CLKMODE_DPLL_DISP) = regVal;

    while(!(HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_IDLEST_DPLL_DISP) &
                         CM_WKUP_CM_IDLEST_DPLL_DISP_ST_DPLL_CLK));
}

/* \brief This function initializes the PER PLL
 * 
 * \param none
 *
 * \return none
 *
 */
void PerPLLInit(void)
{
    volatile unsigned int regVal = 0;

    /* Put the PLL in bypass mode */
    regVal = HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_CLKMODE_DPLL_PER) &
                ~CM_WKUP_CM_CLKMODE_DPLL_PER_DPLL_EN;

    regVal |= CM_WKUP_CM_CLKMODE_DPLL_PER_DPLL_EN_DPLL_MN_BYP_MODE;

    HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_CLKMODE_DPLL_PER) = regVal;

    while(!(HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_IDLEST_DPLL_PER) &
                      CM_WKUP_CM_IDLEST_DPLL_PER_ST_MN_BYPASS));

    HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_CLKSEL_DPLL_PERIPH) &=
                             ~(CM_WKUP_CM_CLKSEL_DPLL_PERIPH_DPLL_MULT |
                                    CM_WKUP_CM_CLKSEL_DPLL_PERIPH_DPLL_DIV);

    /* Set the multipler and divider values for the PLL */
    HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_CLKSEL_DPLL_PERIPH) |=
        ((PERPLL_M << CM_WKUP_CM_CLKSEL_DPLL_PERIPH_DPLL_MULT_SHIFT) |
         (PERPLL_N << CM_WKUP_CM_CLKSEL_DPLL_PERIPH_DPLL_DIV_SHIFT));

    regVal = HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_DIV_M2_DPLL_PER);
    regVal = regVal & ~CM_WKUP_CM_DIV_M2_DPLL_PER_DPLL_CLKOUT_DIV;
    regVal = regVal | PERPLL_M2;

    /* Set the CLKOUT2 divider */
    HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_DIV_M2_DPLL_PER) = regVal;
    
    /* Now LOCK the PLL by enabling it */
    regVal = HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_CLKMODE_DPLL_PER) &
                ~CM_WKUP_CM_CLKMODE_DPLL_PER_DPLL_EN;

    regVal |= CM_WKUP_CM_CLKMODE_DPLL_PER_DPLL_EN;

    HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_CLKMODE_DPLL_PER) = regVal;

    while(!(HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_IDLEST_DPLL_PER) &
                           CM_WKUP_CM_IDLEST_DPLL_PER_ST_DPLL_CLK));

}

/* \brief This function initializes the DDR PLL
 * 
 * \param none
 *
 * \return none
 *
 */
void DDRPLLInit(unsigned int freqMult)
{
    volatile unsigned int regVal = 0;

    /* Put the PLL in bypass mode */
    regVal = HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_CLKMODE_DPLL_DDR) &
                 ~CM_WKUP_CM_CLKMODE_DPLL_DDR_DPLL_EN;

    regVal |= CM_WKUP_CM_CLKMODE_DPLL_MPU_DPLL_EN_DPLL_MN_BYP_MODE;

    HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_CLKMODE_DPLL_DDR) = regVal;

    while(!(HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_IDLEST_DPLL_DDR) &
                      CM_WKUP_CM_IDLEST_DPLL_DDR_ST_MN_BYPASS));

    HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_CLKSEL_DPLL_DDR) &=
                     ~(CM_WKUP_CM_CLKSEL_DPLL_DDR_DPLL_MULT |
                           CM_WKUP_CM_CLKSEL_DPLL_DDR_DPLL_DIV);

    HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_CLKSEL_DPLL_DDR) |=
                     ((freqMult << CM_WKUP_CM_CLKSEL_DPLL_DDR_DPLL_MULT_SHIFT) |
                      (DDRPLL_N << CM_WKUP_CM_CLKSEL_DPLL_DDR_DPLL_DIV_SHIFT));

    regVal = HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_DIV_M2_DPLL_DDR);
    regVal = regVal & ~CM_WKUP_CM_DIV_M2_DPLL_DDR_DPLL_CLKOUT_DIV;
    regVal = regVal | DDRPLL_M2;

    /* Set the CLKOUT2 divider */
    HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_DIV_M2_DPLL_DDR) = regVal;

    /* Now LOCK the PLL by enabling it */
    regVal = HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_CLKMODE_DPLL_DDR) &
                ~CM_WKUP_CM_CLKMODE_DPLL_DDR_DPLL_EN;

    regVal |= CM_WKUP_CM_CLKMODE_DPLL_DDR_DPLL_EN;

    HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_CLKMODE_DPLL_DDR) = regVal;

    while(!(HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_IDLEST_DPLL_DDR) &
                           CM_WKUP_CM_IDLEST_DPLL_DDR_ST_DPLL_CLK));
}

/*
 * \brief This function initializes the MPU PLL
 *
 * \param  none
 *
 * \return none
 */
void MPUPLLInit(unsigned int freqMult)
{
    volatile unsigned int regVal = 0;

    /* Put the PLL in bypass mode */
    regVal = HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_CLKMODE_DPLL_MPU) &
                ~CM_WKUP_CM_CLKMODE_DPLL_MPU_DPLL_EN;

    regVal |= CM_WKUP_CM_CLKMODE_DPLL_MPU_DPLL_EN_DPLL_MN_BYP_MODE;

    HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_CLKMODE_DPLL_MPU) = regVal;

    /* Wait for DPLL to go in to bypass mode */
    while(!(HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_IDLEST_DPLL_MPU) &
                CM_WKUP_CM_IDLEST_DPLL_MPU_ST_MN_BYPASS));

    /* Clear the MULT and DIV field of DPLL_MPU register */
    HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_CLKSEL_DPLL_MPU) &=
                      ~(CM_WKUP_CM_CLKSEL_DPLL_MPU_DPLL_MULT |
                              CM_WKUP_CM_CLKSEL_DPLL_MPU_DPLL_DIV);

    /* Set the multiplier and divider values for the PLL */
    HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_CLKSEL_DPLL_MPU) |=
                     ((freqMult << CM_WKUP_CM_CLKSEL_DPLL_MPU_DPLL_MULT_SHIFT) |
                      (MPUPLL_N << CM_WKUP_CM_CLKSEL_DPLL_MPU_DPLL_DIV_SHIFT));

    regVal = HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_DIV_M2_DPLL_MPU);

    regVal = regVal & ~CM_WKUP_CM_DIV_M2_DPLL_MPU_DPLL_CLKOUT_DIV;

    regVal = regVal | MPUPLL_M2;

    /* Set the CLKOUT2 divider */
    HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_DIV_M2_DPLL_MPU) = regVal;

    /* Now LOCK the PLL by enabling it */
    regVal = HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_CLKMODE_DPLL_MPU) &
                ~CM_WKUP_CM_CLKMODE_DPLL_MPU_DPLL_EN;

    regVal |= CM_WKUP_CM_CLKMODE_DPLL_MPU_DPLL_EN;

    HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_CLKMODE_DPLL_MPU) = regVal;

    while(!(HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_IDLEST_DPLL_MPU) &
                             CM_WKUP_CM_IDLEST_DPLL_MPU_ST_DPLL_CLK));
}

/* \brief This function initializes the interface clock 
 * 
 * \param none
 *
 * \return none
 *
 */
void InterfaceClkInit(void)
{
    HWREG(SOC_CM_PER_REGS + CM_PER_L3_CLKCTRL) |=
                                   CM_PER_L3_CLKCTRL_MODULEMODE_ENABLE;

    while((HWREG(SOC_CM_PER_REGS + CM_PER_L3_CLKCTRL) &
        CM_PER_L3_CLKCTRL_MODULEMODE) != CM_PER_L3_CLKCTRL_MODULEMODE_ENABLE);

    HWREG(SOC_CM_PER_REGS + CM_PER_L4LS_CLKCTRL) |=
                                       CM_PER_L4LS_CLKCTRL_MODULEMODE_ENABLE;

    while((HWREG(SOC_CM_PER_REGS + CM_PER_L4LS_CLKCTRL) &
      CM_PER_L4LS_CLKCTRL_MODULEMODE) != CM_PER_L4LS_CLKCTRL_MODULEMODE_ENABLE);

    HWREG(SOC_CM_PER_REGS + CM_PER_L4FW_CLKCTRL) |=
                                 CM_PER_L4FW_CLKCTRL_MODULEMODE_ENABLE;

    while((HWREG(SOC_CM_PER_REGS + CM_PER_L4FW_CLKCTRL) &
      CM_PER_L4FW_CLKCTRL_MODULEMODE) != CM_PER_L4FW_CLKCTRL_MODULEMODE_ENABLE);

    HWREG(SOC_CM_WKUP_REGS + CM_WKUP_L4WKUP_CLKCTRL) |=
                                          CM_PER_L4FW_CLKCTRL_MODULEMODE_ENABLE;
    while((HWREG(SOC_CM_WKUP_REGS + CM_WKUP_L4WKUP_CLKCTRL) &
                        CM_WKUP_L4WKUP_CLKCTRL_MODULEMODE) !=
                                         CM_PER_L4FW_CLKCTRL_MODULEMODE_ENABLE);

    HWREG(SOC_CM_PER_REGS + CM_PER_L3_INSTR_CLKCTRL) |=
                                      CM_PER_L3_INSTR_CLKCTRL_MODULEMODE_ENABLE;

    while((HWREG(SOC_CM_PER_REGS + CM_PER_L3_INSTR_CLKCTRL) &
                        CM_PER_L3_INSTR_CLKCTRL_MODULEMODE) !=
                        CM_PER_L3_INSTR_CLKCTRL_MODULEMODE_ENABLE);

    HWREG(SOC_CM_PER_REGS + CM_PER_L4HS_CLKCTRL) |=
                              CM_PER_L4HS_CLKCTRL_MODULEMODE_ENABLE;

    while((HWREG(SOC_CM_PER_REGS + CM_PER_L4HS_CLKCTRL) &
                        CM_PER_L4HS_CLKCTRL_MODULEMODE) !=
                  CM_PER_L4HS_CLKCTRL_MODULEMODE_ENABLE);
}

/* \brief This function initializes the power domain transition.
 * 
 * \param none
 *
 * \return none
 *
 */
void PowerDomainTransitionInit(void)
{
    HWREG(SOC_CM_PER_REGS + CM_PER_L3_CLKSTCTRL) |=
                             CM_PER_L3_CLKSTCTRL_CLKTRCTRL_SW_WKUP;

     HWREG(SOC_CM_PER_REGS + CM_PER_L4LS_CLKSTCTRL) |=
                             CM_PER_L4LS_CLKSTCTRL_CLKTRCTRL_SW_WKUP;

    HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CLKSTCTRL) |=
                             CM_WKUP_CLKSTCTRL_CLKTRCTRL_SW_WKUP;

    HWREG(SOC_CM_PER_REGS + CM_PER_L4FW_CLKSTCTRL) |=
                              CM_PER_L4FW_CLKSTCTRL_CLKTRCTRL_SW_WKUP;

    HWREG(SOC_CM_PER_REGS + CM_PER_L3S_CLKSTCTRL) |=
                                CM_PER_L3S_CLKSTCTRL_CLKTRCTRL_SW_WKUP;

}

/*
 * \brief This function sets up various PLLs  
 *
 * \param  none
 *
 * \return none
 */
void PLLInit(void)
{
    MPUPLLInit(oppTable[oppMaxIdx].pllMult);
    CorePLLInit();
    PerPLLInit();
    DDRPLLInit(freqMultDDR);
    InterfaceClkInit();
    PowerDomainTransitionInit();
    DisplayPLLInit();
}

/*
 * \brief Configure I2C0 on which the PMIC is interfaced
 *
 * \param  none
 *
 * \return none
 */
void SetupI2C(void)
{
    I2C0ModuleClkConfig();

    I2CPinMuxSetup(I2C_0);

    /* Put i2c in reset/disabled state */
    I2CMasterDisable(SOC_I2C_0_REGS);

    I2CSoftReset(SOC_I2C_0_REGS);

    /* Disable auto Idle functionality */
    I2CAutoIdleDisable(SOC_I2C_0_REGS);

    /* Configure i2c bus speed to 100khz */
    I2CMasterInitExpClk(SOC_I2C_0_REGS, I2C_SYSTEM_CLOCK, I2C_INTERNAL_CLOCK,
							   I2C_OUTPUT_CLOCK);
    I2CMasterEnable(SOC_I2C_0_REGS);

    while(!I2CSystemStatusGet(SOC_I2C_0_REGS));
}

/*
 * \brief Clear the status of all interrupts 
 *
 * \param  none.
 *
 * \return none
 */

void CleanupInterrupts(void)
{
    I2CMasterIntClearEx(SOC_I2C_0_REGS,  I2C_INTERRUPT_FLAG_TO_CLR);
}

/*
 * \brief Transmits data over I2C0 bus 
 *
 * \param  none
 *
 * \return none
 */
void SetupI2CTransmit(unsigned int dcount)
{
    I2CSetDataCount(SOC_I2C_0_REGS, dcount);

    CleanupInterrupts();

    I2CMasterControl(SOC_I2C_0_REGS, I2C_CFG_MST_TX);

    I2CMasterStart(SOC_I2C_0_REGS);

    while(I2CMasterBusBusy(SOC_I2C_0_REGS) == 0);

    while((I2C_INT_TRANSMIT_READY == (I2CMasterIntRawStatus(SOC_I2C_0_REGS)
                                     & I2C_INT_TRANSMIT_READY)) && dcount--)
    {
        I2CMasterDataPut(SOC_I2C_0_REGS, dataToSlave[tCount++]);

        I2CMasterIntClearEx(SOC_I2C_0_REGS, I2C_INT_TRANSMIT_READY);
    }

    I2CMasterStop(SOC_I2C_0_REGS);

    while(0 == (I2CMasterIntRawStatus(SOC_I2C_0_REGS) & I2C_INT_STOP_CONDITION));

    I2CMasterIntClearEx(SOC_I2C_0_REGS, I2C_INT_STOP_CONDITION);

}

/*
 * \brief Receives data over I2C0 bus 
 *
 * \param  dcount - Number of bytes to receive.
 *
 * \return none
 */

void SetupReception(unsigned int dcount)
{
    I2CSetDataCount(SOC_I2C_0_REGS, 1);

    CleanupInterrupts();

    I2CMasterControl(SOC_I2C_0_REGS, I2C_CFG_MST_TX);

    I2CMasterStart(SOC_I2C_0_REGS);

    while(I2CMasterBusBusy(SOC_I2C_0_REGS) == 0);

    I2CMasterDataPut(SOC_I2C_0_REGS, dataToSlave[tCount]);

    I2CMasterIntClearEx(SOC_I2C_0_REGS, I2C_INT_TRANSMIT_READY);

    while(0 == (I2CMasterIntRawStatus(SOC_I2C_0_REGS) & I2C_INT_ADRR_READY_ACESS));

    I2CSetDataCount(SOC_I2C_0_REGS, dcount);

    CleanupInterrupts();

    I2CMasterControl(SOC_I2C_0_REGS, I2C_CFG_MST_RX);

    I2CMasterStart(SOC_I2C_0_REGS);

    /* Wait till the bus if free */
    while(I2CMasterBusBusy(SOC_I2C_0_REGS) == 0);

    /* Read the data from slave of dcount */
    while((dcount--))
    {
        while(0 == (I2CMasterIntRawStatus(SOC_I2C_0_REGS) & I2C_INT_RECV_READY));

        dataFromSlave[rCount++] = I2CMasterDataGet(SOC_I2C_0_REGS);

        I2CMasterIntClearEx(SOC_I2C_0_REGS, I2C_INT_RECV_READY);
    }

    I2CMasterStop(SOC_I2C_0_REGS);

    while(0 == (I2CMasterIntRawStatus(SOC_I2C_0_REGS) & I2C_INT_STOP_CONDITION));

    I2CMasterIntClearEx(SOC_I2C_0_REGS, I2C_INT_STOP_CONDITION);
}

/*
 * \brief Generic function that can read a TPS65217 register
 *
 * \param regOffset -  Source register address
 *
 * \return dest     -  Place holder for read bytes.
 */
#ifdef beaglebone

void TPS65217RegRead(unsigned char regOffset, unsigned char* dest)
{
    dataToSlave[0] = regOffset;
    tCount = 0;

    SetupReception(1);

    *dest = dataFromSlave[0];
}

/**
 *  \brief            - Generic function that can write a TPS65217 PMIC
 *                      register or bit field regardless of protection
 *                      level.
 *
 * \param prot_level  - Register password protection.
 *                      use PROT_LEVEL_NONE, PROT_LEVEL_1, or PROT_LEVEL_2
 * \param regOffset:  - Register address to write.
 *
 * \param dest_val    - Value to write.
 *
 * \param mask        - Bit mask (8 bits) to be applied.  Function will only
 *                      change bits that are set in the bit mask.
 *
 * \return:            None.
 */
void TPS65217RegWrite(unsigned char port_level, unsigned char regOffset,
                      unsigned char dest_val, unsigned char mask)
{
    unsigned char read_val;
    unsigned xor_reg;

    dataToSlave[0] = regOffset;
    tCount = 0;
    rCount = 0;

    if(mask != MASK_ALL_BITS)
    {
         SetupReception(1);

         read_val = dataFromSlave[0];
         read_val &= (~mask);
         read_val |= (dest_val & mask);
         dest_val = read_val;
    }

    if(port_level > 0)
    {
         xor_reg = regOffset ^ PASSWORD_UNLOCK;

         dataToSlave[0] = PASSWORD;
         dataToSlave[1] = xor_reg;
         tCount = 0;

         SetupI2CTransmit(2);
    }

    dataToSlave[0] = regOffset;
    dataToSlave[1] = dest_val;
    tCount = 0;

    SetupI2CTransmit(2);

    if(port_level == PROT_LEVEL_2)
    {
         dataToSlave[0] = PASSWORD;
         dataToSlave[1] = xor_reg;
         tCount = 0;

         SetupI2CTransmit(2);

         dataToSlave[0] = regOffset;
         dataToSlave[1] = dest_val;
         tCount = 0;

         SetupI2CTransmit(2);
    }
}

/**
 *  \brief              - Controls output voltage setting for the DCDC1,
 *                        DCDC2, or DCDC3 control registers in the PMIC.
 *
 * \param  dc_cntrl_reg   DCDC Control Register address.
 *                        Must be DEFDCDC1, DEFDCDC2, or DEFDCDC3.
 *
 * \param  volt_sel       Register value to set.  See PMIC TRM for value set.
 *
 * \return:               None.
 */
void TPS65217VoltageUpdate(unsigned char dc_cntrl_reg, unsigned char volt_sel)
{
    /* set voltage level */
    TPS65217RegWrite(PROT_LEVEL_2, dc_cntrl_reg, volt_sel, MASK_ALL_BITS);

    /* set GO bit to initiate voltage transition */
    TPS65217RegWrite(PROT_LEVEL_2, DEFSLEW, DCDC_GO, DCDC_GO);
}

#else

/**
 *  \brief                 - Configure vdd2 for various parameters such as 
 *                           Multiplier, Maximum Load Current etc .
 *
 * \param  opVolMultiplier - Multiplier.
 *
 * \param  maxLoadCurrent  - Maximum Load Current.
 * 
 * \param  timeStep        - Time step - voltage change per us(micro sec).
 *  
 * \param  supplyState     - Supply state (on (high/low power mode), off)
 *
 * \return:               None.
 */

void ConfigureVdd2(unsigned int opVolMultiplier, unsigned maxLoadCurrent,
                   unsigned int timeStep, unsigned int supplyState)
{
    dataToSlave[0] = VDD2_REG;
    dataToSlave[1] = ((opVolMultiplier << PMIC_VDD2_REG_VGAIN_SEL_SHIFT) |
		              (maxLoadCurrent << PMIC_VDD2_REG_ILMAX_SHIFT)      |
		              (timeStep << PMIC_VDD2_REG_TSTEP_SHIFT)            |
		              (supplyState << PMIC_VDD2_REG_ST_SHIFT));
    tCount = 0;
    rCount = 0;
    SetupI2CTransmit(2);
}


/**
 *  \brief Select the VDD2 value. VDD2_OP_REG or VDD2_SR_REG.
 *
 * \param  vddSource  - VDD2 value.
 *
 * \return None.
 */
void SelectVdd2Source(unsigned int vddSource)
{
    /*	Read reg value	*/
    dataToSlave[0] = VDD2_OP_REG;
    dataFromSlave[0] = 0; // clear receive buffer
    dataFromSlave[1] = 0;
    rCount = 0;
    tCount = 0;
    SetupReception(1);

    /*	Modify reg value	*/
    vddSource = (dataFromSlave[0] & (~PMIC_VDD2_OP_REG_CMD)) |
		 (vddSource << PMIC_VDD2_OP_REG_CMD_SHIFT);

	/*	Write reg value	*/
    dataToSlave[0] = VDD2_OP_REG;
    dataToSlave[1] = vddSource;
    tCount = 0;
    rCount = 0;

    SetupI2CTransmit(2);
}

/**
 *  \brief set VDD2_OP voltage value.
 *
 * \param  opVolSelector  - VDD2_OP voltage value.
 *
 * \return None.
 */
void SetVdd2OpVoltage(unsigned int opVolSelector)
{
    /*	Read reg value	*/
    dataToSlave[0] = VDD2_OP_REG;
    dataFromSlave[0] = 0; // clear receive buffer
    dataFromSlave[1] = 0;
    rCount = 0;
    tCount = 0;
    SetupReception(1);

    /*	Modify reg value	*/
    opVolSelector = (dataFromSlave[0] & (~PMIC_VDD2_OP_REG_SEL)) |
		            (opVolSelector << PMIC_VDD2_OP_REG_SEL_SHIFT);

    /*	Write reg value	*/
    dataToSlave[0] = VDD2_OP_REG;
    dataToSlave[1] = opVolSelector;
    tCount = 0;
    rCount = 0;
    SetupI2CTransmit(2);

     /*	Read reg value to verify */
    dataToSlave[0] = VDD2_OP_REG;
    dataFromSlave[0] = 0; // clear receive buffer
    dataFromSlave[1] = 0;
    rCount = 0;
    tCount = 0;

    SetupReception(1);

    while((dataFromSlave[0] & PMIC_VDD2_OP_REG_SEL) != 
          (opVolSelector << PMIC_VDD2_OP_REG_SEL_SHIFT));
}

/**
 *  \brief set VDD2_SR voltage value
 *
 * \param  opVolSelector  - VDD2_SR voltage value.
 *
 * \return None.
 */
void SetVdd2SrVoltage(unsigned int opVolSelector)
{
    /*	Write reg value	*/
    dataToSlave[0] = VDD2_SR_REG;
    dataToSlave[1] = opVolSelector;
    tCount = 0;
    SetupI2CTransmit(2);	
}

/**
 *  \brief Select I2C interface whether SR I2C or Control I2C
 *
 * \param  i2cInstance  - I2c instance to select.
 *
 * \return None.
 */
void SelectI2CInstance(unsigned int i2cInstance)
{
    /*	Read reg value	*/
    dataToSlave[0] = DEVCTRL_REG;
    dataFromSlave[0] = 0; // clear receive buffer
    dataFromSlave[1] = 0;
    rCount = 0;
    tCount = 0;
    SetupReception(1);

    /*	Modify reg value */
    i2cInstance = (dataFromSlave[0] & (~PMIC_DEVCTRL_REG_SR_CTL_I2C_SEL)) |
                  (i2cInstance << PMIC_DEVCTRL_REG_SR_CTL_I2C_SEL_SHIFT);

    /*	Write reg value	*/
    dataToSlave[0] = DEVCTRL_REG;
    dataToSlave[1] = i2cInstance;
    tCount = 0;
    rCount = 0;

    SetupI2CTransmit(2);
}

/**
 *  \brief                 - Configure vdd1 for various parameters such as 
 *                           Multiplier, Maximum Load Current etc .
 *
 * \param  opVolMultiplier - Multiplier.
 *
 * \param  maxLoadCurrent  - Maximum Load Current.
 * 
 * \param  timeStep        - Time step - voltage change per us(micro sec).
 *  
 * \param  supplyState     - Supply state (on (high/low power mode), off)
 *
 * \return:               None.
 */
void ConfigureVdd1(unsigned int opVolMultiplier, unsigned maxLoadCurrent,
                   unsigned int timeStep, unsigned int supplyState)
{
    dataToSlave[0] = VDD1_REG;
    dataToSlave[1] = ((opVolMultiplier << PMIC_VDD1_REG_VGAIN_SEL_SHIFT) |
		      (maxLoadCurrent << PMIC_VDD1_REG_ILMAX_SHIFT) |
		      (timeStep << PMIC_VDD1_REG_TSTEP_SHIFT) |
		      (supplyState << PMIC_VDD1_REG_ST_SHIFT) );
    tCount = 0;
    rCount = 0;
    SetupI2CTransmit(2);
}

/**
 *  \brief Select the VDD1 value. VDD1_OP_REG or VDD1_SR_REG.
 *
 * \param  vddSource  - VDD2 value.
 *
 * \return None.
 */
void SelectVdd1Source(unsigned int vddSource)
{
    /*	Read reg value	*/
    dataToSlave[0] = VDD1_OP_REG;
    dataFromSlave[0] = 0; // clear receive buffer
    dataFromSlave[1] = 0;
    rCount = 0;
    tCount = 0;
    SetupReception(1);

    /*	Modify reg value */
    vddSource = (dataFromSlave[0] & (~PMIC_VDD1_OP_REG_CMD)) |
		 (vddSource << PMIC_VDD1_OP_REG_CMD_SHIFT);

     /*	Write reg value	*/
    dataToSlave[0] = VDD1_OP_REG;
    dataToSlave[1] = vddSource;
    tCount = 0;
    rCount = 0;
    SetupI2CTransmit(2);
}

#endif

/**
 *  \brief set VDD1_OP voltage value.
 *
 * \param  opVolSelector  - VDD2_OP voltage value.
 *
 * \return None.
 */
void SetVdd1OpVoltage(unsigned int opVolSelector)
{
#ifdef beaglebone

    /* Set DCDC2 (MPU) voltage */
    TPS65217VoltageUpdate(DEFDCDC2, opVolSelector);

#elif  defined (evmAM335x) || defined (evmskAM335x)

    /*	Read reg value	*/
    dataToSlave[0] = VDD1_OP_REG;
    dataFromSlave[0] = 0; // clear receive buffer
    dataFromSlave[1] = 0;
    rCount = 0;
    tCount = 0;
    SetupReception(1);

    /*	Modify reg value */
    opVolSelector = (dataFromSlave[0] & (~PMIC_VDD1_OP_REG_SEL)) |
		    (opVolSelector << PMIC_VDD1_OP_REG_SEL_SHIFT);

    /*	Write reg value	*/
    dataToSlave[0] = VDD1_OP_REG;
    dataToSlave[1] = opVolSelector;
    tCount = 0;
    rCount = 0;
    SetupI2CTransmit(2);

    /*	Read reg value to verify */
    dataToSlave[0] = VDD1_OP_REG;
    dataFromSlave[0] = 0; // clear receive buffer
    dataFromSlave[1] = 0;
    rCount = 0;
    tCount = 0;
    SetupReception(1);

    while((dataFromSlave[0] & PMIC_VDD1_OP_REG_SEL) != 
          (opVolSelector << PMIC_VDD1_OP_REG_SEL_SHIFT));

#endif
}

/*
 * \brief Configures the VDD OP voltage. 
 *
 * \param  none.
 *
 * \return none
 */

void ConfigVddOpVoltage(void)
{
    SetupI2C();

#ifdef beaglebone

    unsigned char pmic_status = 0;

    /* Configure PMIC slave address */
    I2CMasterSlaveAddrSet(SOC_I2C_0_REGS, PMIC_TPS65217_I2C_SLAVE_ADDR);

    TPS65217RegRead(STATUS, &pmic_status);

    /* Increase USB current limit to 1300mA */
    TPS65217RegWrite(PROT_LEVEL_NONE, POWER_PATH, USB_INPUT_CUR_LIMIT_1300MA,
                       USB_INPUT_CUR_LIMIT_MASK);

    /* Set DCDC2 (MPU) voltage to 1.275V */
    TPS65217VoltageUpdate(DEFDCDC2, DCDC_VOLT_SEL_1275MV);

    /* Set LDO3, LDO4 output voltage to 3.3V */
    TPS65217RegWrite(PROT_LEVEL_2, DEFLS1, LDO_VOLTAGE_OUT_3_3, LDO_MASK);


    TPS65217RegWrite(PROT_LEVEL_2, DEFLS2, LDO_VOLTAGE_OUT_3_3, LDO_MASK);

#elif  defined (evmAM335x) || defined (evmskAM335x)

    /* Configure PMIC slave address */
    I2CMasterSlaveAddrSet(SOC_I2C_0_REGS, PMIC_CNTL_I2C_SLAVE_ADDR);

	/* Select SR I2C(0) */
    SelectI2CInstance(PMIC_DEVCTRL_REG_SR_CTL_I2C_SEL_CTL_I2C);

    /* Configure vdd1- need to validate these parameters */
    ConfigureVdd1(PMIC_VDD1_REG_VGAIN_SEL_X1, PMIC_VDD1_REG_ILMAX_1_5_A,
	              PMIC_VDD1_REG_TSTEP_12_5, PMIC_VDD1_REG_ST_ON_HI_POW);

    /* Select the source for VDD1 control */
    SelectVdd1Source(PMIC_VDD1_OP_REG_CMD_OP);

#else

    #error Unsupported EVM !!

#endif
}

/*
 * \brief This function sets up the DDR PHY
 *
 * \param  none
 *
 * \return none
 */
#ifdef am335x
static void DDR2PhyInit(void)
{
    /* Enable VTP */
    HWREG(SOC_CONTROL_REGS + CONTROL_VTP_CTRL) |= CONTROL_VTP_CTRL_ENABLE;
    HWREG(SOC_CONTROL_REGS + CONTROL_VTP_CTRL) &= ~CONTROL_VTP_CTRL_CLRZ;
    HWREG(SOC_CONTROL_REGS + CONTROL_VTP_CTRL) |= CONTROL_VTP_CTRL_CLRZ;
    while((HWREG(SOC_CONTROL_REGS + CONTROL_VTP_CTRL) & CONTROL_VTP_CTRL_READY) !=
                CONTROL_VTP_CTRL_READY);

    /* DDR PHY CMD0 Register configuration */
    HWREG(CMD0_SLAVE_RATIO_0)   = DDR2_CMD0_SLAVE_RATIO_0;
    HWREG(CMD0_SLAVE_FORCE_0)   = DDR2_CMD0_SLAVE_FORCE_0;
    HWREG(CMD0_SLAVE_DELAY_0)   = DDR2_CMD0_SLAVE_DELAY_0;
    HWREG(CMD0_LOCK_DIFF_0)     = DDR2_CMD0_LOCK_DIFF_0;
    HWREG(CMD0_INVERT_CLKOUT_0) = DDR2_CMD0_INVERT_CLKOUT_0;

    /* DDR PHY CMD1 Register configuration */
    HWREG(CMD1_SLAVE_RATIO_0)   = DDR2_CMD1_SLAVE_RATIO_0;
    HWREG(CMD1_SLAVE_FORCE_0)   =  DDR2_CMD1_SLAVE_FORCE_0;
    HWREG(CMD1_SLAVE_DELAY_0)   = DDR2_CMD1_SLAVE_DELAY_0;
    HWREG(CMD1_LOCK_DIFF_0)     = DDR2_CMD1_LOCK_DIFF_0;
    HWREG(CMD1_INVERT_CLKOUT_0) = DDR2_CMD1_INVERT_CLKOUT_0;

    /* DDR PHY CMD2 Register configuration */
    HWREG(CMD2_SLAVE_RATIO_0)   = DDR2_CMD2_SLAVE_RATIO_0;
    HWREG(CMD2_SLAVE_FORCE_0)   = DDR2_CMD2_SLAVE_FORCE_0;
    HWREG(CMD2_SLAVE_DELAY_0)   = DDR2_CMD2_SLAVE_DELAY_0;
    HWREG(CMD2_LOCK_DIFF_0)     = DDR2_CMD2_LOCK_DIFF_0;
    HWREG(CMD2_INVERT_CLKOUT_0) = DDR2_CMD2_INVERT_CLKOUT_0;

    /* DATA macro configuration */
    HWREG(DATA0_RD_DQS_SLAVE_RATIO_0)  = DDR2_DATA0_RD_DQS_SLAVE_RATIO_0;
    HWREG(DATA0_WR_DQS_SLAVE_RATIO_0)  = DDR2_DATA0_WR_DQS_SLAVE_RATIO_0;
    HWREG(DATA0_FIFO_WE_SLAVE_RATIO_0) = DDR2_DATA0_FIFO_WE_SLAVE_RATIO_0;
    HWREG(DATA0_WR_DATA_SLAVE_RATIO_0) = DDR2_DATA0_WR_DATA_SLAVE_RATIO_0;

    HWREG(DATA1_RD_DQS_SLAVE_RATIO_0)  = DDR2_DATA1_RD_DQS_SLAVE_RATIO_0;
    HWREG(DATA1_WR_DQS_SLAVE_RATIO_0)  = DDR2_DATA1_WR_DQS_SLAVE_RATIO_0;
    HWREG(DATA1_FIFO_WE_SLAVE_RATIO_0) = DDR2_DATA1_FIFO_WE_SLAVE_RATIO_0;
    HWREG(DATA1_WR_DATA_SLAVE_RATIO_0) = DDR2_DATA1_WR_DATA_SLAVE_RATIO_0;
}
/*
 * \brief This function sets up the DDR PHY
 *
 * \param  none
 *
 * \return none
 */
static void DDR3PhyInit(void)
{
    /* Enable VTP */
    HWREG(SOC_CONTROL_REGS + CONTROL_VTP_CTRL) |= CONTROL_VTP_CTRL_ENABLE;
    HWREG(SOC_CONTROL_REGS + CONTROL_VTP_CTRL) &= ~CONTROL_VTP_CTRL_CLRZ;
    HWREG(SOC_CONTROL_REGS + CONTROL_VTP_CTRL) |= CONTROL_VTP_CTRL_CLRZ;
    while((HWREG(SOC_CONTROL_REGS + CONTROL_VTP_CTRL) & CONTROL_VTP_CTRL_READY) !=
                CONTROL_VTP_CTRL_READY);

    /* DDR PHY CMD0 Register configuration */
    HWREG(CMD0_SLAVE_RATIO_0)   = DDR3_CMD0_SLAVE_RATIO_0;
    HWREG(CMD0_INVERT_CLKOUT_0) = DDR3_CMD0_INVERT_CLKOUT_0;

    /* DDR PHY CMD1 Register configuration */
    HWREG(CMD1_SLAVE_RATIO_0)   = DDR3_CMD1_SLAVE_RATIO_0;
    HWREG(CMD1_INVERT_CLKOUT_0) = DDR3_CMD1_INVERT_CLKOUT_0;

    /* DDR PHY CMD2 Register configuration */
    HWREG(CMD2_SLAVE_RATIO_0)   = DDR3_CMD2_SLAVE_RATIO_0;
    HWREG(CMD2_INVERT_CLKOUT_0) = DDR3_CMD2_INVERT_CLKOUT_0;

    /* DATA macro configuration */
    HWREG(DATA0_RD_DQS_SLAVE_RATIO_0)  = DDR3_DATA0_RD_DQS_SLAVE_RATIO_0;
    HWREG(DATA0_WR_DQS_SLAVE_RATIO_0)  = DDR3_DATA0_WR_DQS_SLAVE_RATIO_0;
    HWREG(DATA0_FIFO_WE_SLAVE_RATIO_0) = DDR3_DATA0_FIFO_WE_SLAVE_RATIO_0;
    HWREG(DATA0_WR_DATA_SLAVE_RATIO_0) = DDR3_DATA0_WR_DATA_SLAVE_RATIO_0;
    HWREG(DATA1_RD_DQS_SLAVE_RATIO_0)  = DDR3_DATA0_RD_DQS_SLAVE_RATIO_1;
    HWREG(DATA1_WR_DQS_SLAVE_RATIO_0)  = DDR3_DATA0_WR_DQS_SLAVE_RATIO_1;
    HWREG(DATA1_FIFO_WE_SLAVE_RATIO_0) = DDR3_DATA0_FIFO_WE_SLAVE_RATIO_1;
    HWREG(DATA1_WR_DATA_SLAVE_RATIO_0) = DDR3_DATA0_WR_DATA_SLAVE_RATIO_1;

}


/* \brief This function initializes the DDR2
 * 
 * \param none
 *
 * \return none
 *
 */
void DDR3Init(void)
{
    /* DDR3 Phy Initialization */
    DDR3PhyInit();

    HWREG(SOC_CONTROL_REGS + CONTROL_DDR_CMD_IOCTRL(0)) =
                                                 DDR3_CONTROL_DDR_CMD_IOCTRL_0;
    HWREG(SOC_CONTROL_REGS + CONTROL_DDR_CMD_IOCTRL(1)) =
                                                 DDR3_CONTROL_DDR_CMD_IOCTRL_1;
    HWREG(SOC_CONTROL_REGS + CONTROL_DDR_CMD_IOCTRL(2)) =
                                                 DDR3_CONTROL_DDR_CMD_IOCTRL_2;
    HWREG(SOC_CONTROL_REGS + CONTROL_DDR_DATA_IOCTRL(0)) =
                                                 DDR3_CONTROL_DDR_DATA_IOCTRL_0;
    HWREG(SOC_CONTROL_REGS + CONTROL_DDR_DATA_IOCTRL(1)) =
                                                 DDR3_CONTROL_DDR_DATA_IOCTRL_1;

    /* IO to work for DDR3 */
    HWREG(SOC_CONTROL_REGS + CONTROL_DDR_IO_CTRL) &= DDR3_CONTROL_DDR_IO_CTRL;

    HWREG(SOC_CONTROL_REGS + CONTROL_DDR_CKE_CTRL) |= CONTROL_DDR_CKE_CTRL_DDR_CKE_CTRL;

    HWREG(SOC_EMIF_0_REGS + EMIF_DDR_PHY_CTRL_1) = DDR3_EMIF_DDR_PHY_CTRL_1;

    /* Dynamic Power Down */
    if((DEVICE_VERSION_2_0 == deviceVersion) ||
       (DEVICE_VERSION_2_1 == deviceVersion))
    {
        HWREG(SOC_EMIF_0_REGS + EMIF_DDR_PHY_CTRL_1) |=
                                              DDR3_EMIF_DDR_PHY_CTRL_1_DY_PWRDN;
    }

    HWREG(SOC_EMIF_0_REGS + EMIF_DDR_PHY_CTRL_1_SHDW) =
                                                 DDR3_EMIF_DDR_PHY_CTRL_1_SHDW;

    /* Dynamic Power Down */
    if((DEVICE_VERSION_2_0 == deviceVersion) ||
       (DEVICE_VERSION_2_1 == deviceVersion))
    {
        HWREG(SOC_EMIF_0_REGS + EMIF_DDR_PHY_CTRL_1_SHDW) |=
                                         DDR3_EMIF_DDR_PHY_CTRL_1_SHDW_DY_PWRDN;
    }

    HWREG(SOC_EMIF_0_REGS + EMIF_DDR_PHY_CTRL_2) = DDR3_EMIF_DDR_PHY_CTRL_2;

    HWREG(SOC_EMIF_0_REGS + EMIF_SDRAM_TIM_1)      = DDR3_EMIF_SDRAM_TIM_1;
    HWREG(SOC_EMIF_0_REGS + EMIF_SDRAM_TIM_1_SHDW) = DDR3_EMIF_SDRAM_TIM_1_SHDW;
    HWREG(SOC_EMIF_0_REGS + EMIF_SDRAM_TIM_2)      = DDR3_EMIF_SDRAM_TIM_2;
    HWREG(SOC_EMIF_0_REGS + EMIF_SDRAM_TIM_2_SHDW) = DDR3_EMIF_SDRAM_TIM_2_SHDW;
    HWREG(SOC_EMIF_0_REGS + EMIF_SDRAM_TIM_3)      = DDR3_EMIF_SDRAM_TIM_3;
    HWREG(SOC_EMIF_0_REGS + EMIF_SDRAM_TIM_3_SHDW) = DDR3_EMIF_SDRAM_TIM_3_SHDM;

    HWREG(SOC_EMIF_0_REGS + EMIF_SDRAM_REF_CTRL)   = DDR3_EMIF_SDRAM_REF_CTRL_VAL1;
    HWREG(SOC_EMIF_0_REGS + EMIF_SDRAM_REF_CTRL_SHDW) =
                                                 DDR3_EMIF_SDRAM_REF_CTRL_SHDW_VAL1;

    HWREG(SOC_EMIF_0_REGS + EMIF_ZQ_CONFIG)     = DDR3_EMIF_ZQ_CONFIG_VAL;
    HWREG(SOC_EMIF_0_REGS + EMIF_SDRAM_CONFIG)     = DDR3_EMIF_SDRAM_CONFIG;
	
    /* The CONTROL_SECURE_EMIF_SDRAM_CONFIG register exports SDRAM configuration 
       information to the EMIF */
    HWREG(SOC_CONTROL_REGS + CONTROL_SECURE_EMIF_SDRAM_CONFIG) = DDR3_EMIF_SDRAM_CONFIG;

}

/* \brief This function initializes the DDR2
 * 
 * \param none
 *
 * \return none
 *
 */
void DDR2Init(void)
{
    volatile unsigned int delay = 5000;

    /* DDR2 Phy Initialization */
    DDR2PhyInit();

    HWREG(SOC_CONTROL_REGS + CONTROL_DDR_CMD_IOCTRL(0)) =
                                                 DDR2_CONTROL_DDR_CMD_IOCTRL_0;
    HWREG(SOC_CONTROL_REGS + CONTROL_DDR_CMD_IOCTRL(1)) =
                                                 DDR2_CONTROL_DDR_CMD_IOCTRL_1;
    HWREG(SOC_CONTROL_REGS + CONTROL_DDR_CMD_IOCTRL(2)) =
                                                 DDR2_CONTROL_DDR_CMD_IOCTRL_2;
    HWREG(SOC_CONTROL_REGS + CONTROL_DDR_DATA_IOCTRL(0)) =
                                                 DDR2_CONTROL_DDR_DATA_IOCTRL_0;
    HWREG(SOC_CONTROL_REGS + CONTROL_DDR_DATA_IOCTRL(1)) =
                                                 DDR2_CONTROL_DDR_DATA_IOCTRL_1;

    /* IO to work for DDR2 */
    HWREG(SOC_CONTROL_REGS + CONTROL_DDR_IO_CTRL) &= DDR2_CONTROL_DDR_IO_CTRL;

    HWREG(SOC_CONTROL_REGS + CONTROL_DDR_CKE_CTRL) |= CONTROL_DDR_CKE_CTRL_DDR_CKE_CTRL;


    HWREG(SOC_EMIF_0_REGS + EMIF_DDR_PHY_CTRL_1) = DDR2_EMIF_DDR_PHY_CTRL_1;

    /* Dynamic Power Down */
    if((DEVICE_VERSION_2_0 == deviceVersion) ||
       (DEVICE_VERSION_2_1 == deviceVersion))
    {
        HWREG(SOC_EMIF_0_REGS + EMIF_DDR_PHY_CTRL_1) |=
                                              DDR2_EMIF_DDR_PHY_CTRL_1_DY_PWRDN;
    }

    HWREG(SOC_EMIF_0_REGS + EMIF_DDR_PHY_CTRL_1_SHDW) =
                                                 DDR2_EMIF_DDR_PHY_CTRL_1_SHDW;

    /* Dynamic Power Down */
    if((DEVICE_VERSION_2_0 == deviceVersion) ||
       (DEVICE_VERSION_2_1 == deviceVersion))
    {
        HWREG(SOC_EMIF_0_REGS + EMIF_DDR_PHY_CTRL_1_SHDW) |=
                                         DDR2_EMIF_DDR_PHY_CTRL_1_SHDW_DY_PWRDN;
    }

    HWREG(SOC_EMIF_0_REGS + EMIF_DDR_PHY_CTRL_2) = DDR2_EMIF_DDR_PHY_CTRL_2;

    HWREG(SOC_EMIF_0_REGS + EMIF_SDRAM_TIM_1)      =  DDR2_EMIF_SDRAM_TIM_1;
    HWREG(SOC_EMIF_0_REGS + EMIF_SDRAM_TIM_1_SHDW) = DDR2_EMIF_SDRAM_TIM_1_SHDW;
    HWREG(SOC_EMIF_0_REGS + EMIF_SDRAM_TIM_2)      = DDR2_EMIF_SDRAM_TIM_2;
    HWREG(SOC_EMIF_0_REGS + EMIF_SDRAM_TIM_2_SHDW) = DDR2_EMIF_SDRAM_TIM_2_SHDW;
    HWREG(SOC_EMIF_0_REGS + EMIF_SDRAM_TIM_3)      = DDR2_EMIF_SDRAM_TIM_3;
    HWREG(SOC_EMIF_0_REGS + EMIF_SDRAM_TIM_3_SHDW) = DDR2_EMIF_SDRAM_TIM_3_SHDM;
    HWREG(SOC_EMIF_0_REGS + EMIF_SDRAM_CONFIG)     = DDR2_EMIF_SDRAM_CONFIG;
    HWREG(SOC_EMIF_0_REGS + EMIF_SDRAM_REF_CTRL)   = DDR2_EMIF_SDRAM_REF_CTRL_VAL1;
    HWREG(SOC_EMIF_0_REGS + EMIF_SDRAM_REF_CTRL_SHDW) =
                                                 DDR2_EMIF_SDRAM_REF_CTRL_SHDW_VAL1;

    while(delay--);

    HWREG(SOC_EMIF_0_REGS + EMIF_SDRAM_REF_CTRL) = DDR2_EMIF_SDRAM_REF_CTRL_VAL2;
    HWREG(SOC_EMIF_0_REGS + EMIF_SDRAM_REF_CTRL_SHDW) =
                                                 DDR2_EMIF_SDRAM_REF_CTRL_SHDW_VAL2;

    HWREG(SOC_EMIF_0_REGS + EMIF_SDRAM_CONFIG)   = DDR2_EMIF_SDRAM_CONFIG;
	
    /* The CONTROL_SECURE_EMIF_SDRAM_CONFIG register exports SDRAM configuration 
       information to the EMIF */
    HWREG(SOC_CONTROL_REGS + CONTROL_SECURE_EMIF_SDRAM_CONFIG) = DDR2_EMIF_SDRAM_CONFIG;

}
#else
#error "---------------------------------------------------"
#error "           UNSUPPORTED MEMORY CONFIGURATION        "
#error "---------------------------------------------------"
#endif

/* \brief This function initializes the EMIF
 * 
 * \param none
 *
 * \return none
 *
 */
void EMIFInit(void)
{
    volatile unsigned int regVal;

    /* Enable the clocks for EMIF */
    regVal = HWREG(SOC_CM_PER_REGS + CM_PER_EMIF_FW_CLKCTRL) &
                ~(CM_PER_EMIF_FW_CLKCTRL_MODULEMODE);

    regVal |= CM_PER_EMIF_FW_CLKCTRL_MODULEMODE_ENABLE;

    HWREG(SOC_CM_PER_REGS + CM_PER_EMIF_FW_CLKCTRL) = regVal;

    regVal = HWREG(SOC_CM_PER_REGS + CM_PER_EMIF_CLKCTRL) &
                ~(CM_PER_EMIF_CLKCTRL_MODULEMODE);

    regVal |= CM_PER_EMIF_CLKCTRL_MODULEMODE_ENABLE;

    HWREG(SOC_CM_PER_REGS + CM_PER_EMIF_CLKCTRL) = regVal;

    while((HWREG(SOC_CM_PER_REGS + CM_PER_L3_CLKSTCTRL) &
          (CM_PER_L3_CLKSTCTRL_CLKACTIVITY_EMIF_GCLK |
           CM_PER_L3_CLKSTCTRL_CLKACTIVITY_L3_GCLK)) !=
          (CM_PER_L3_CLKSTCTRL_CLKACTIVITY_EMIF_GCLK |
           CM_PER_L3_CLKSTCTRL_CLKACTIVITY_L3_GCLK));
}

#if defined(NAND)

/******************************************************************************
*                                                                             *
*                                                                             *
* \brief  Function to initalize the GPMC NAND timing and base addr info.      *
*                                                                             *
* \param  nandTimimgInfo : Pointer to structure containing                    *
*                          NAND timing info.                                  *
*                                                                             *
* \return none.                                                               *
*                                                                             *
******************************************************************************/
static void NANDTimingInfoInit(void *TimingInfo)
{

    GPMCNANDTimingInfo_t *nandTimingInfo;

    nandTimingInfo = (GPMCNANDTimingInfo_t * )TimingInfo;

    nandTimingInfo->CSWrOffTime               = NAND_CSWROFFTIME;
    nandTimingInfo->CSRdOffTime               = NAND_CSRDOFFTIME;
    nandTimingInfo->CSExtDelayFlag            = GPMC_CS_EXTRA_NODELAY;
    nandTimingInfo->CSOnTime                  = NAND_CSONTIME;

    nandTimingInfo->ADVAADMuxWrOffTime        = NAND_ADVAADMUXWROFFTIME;
    nandTimingInfo->ADVAADMuxRdOffTime        = NAND_ADVAADMUXRDOFFTIME;
    nandTimingInfo->ADVWrOffTime              = NAND_ADVWROFFTIME;
    nandTimingInfo->ADVRdOffTime              = NAND_ADVRDOFFTIME;
    nandTimingInfo->ADVExtDelayFlag           = GPMC_ADV_EXTRA_NODELAY;
    nandTimingInfo->ADVAADMuxOnTime           = NAND_ADVAADMUXONTIME;
    nandTimingInfo->ADVOnTime                 = NAND_ADVONTIME;

    nandTimingInfo->WEOffTime                 = NAND_WEOFFTIME;
    nandTimingInfo->WEExtDelayFlag            = GPMC_WE_EXTRA_NODELAY;
    nandTimingInfo->WEOnTime                  = NAND_WEONTIME;
    nandTimingInfo->OEAADMuxOffTime           = NAND_OEAADMUXOFFTIME;
    nandTimingInfo->OEOffTime                 = NAND_OEOFFTIME;
    nandTimingInfo->OEExtDelayFlag            = GPMC_OE_EXTRA_NODELAY;
    nandTimingInfo->OEAADMuxOnTime            = NAND_OEAADMUXONTIME;
    nandTimingInfo->OEOnTime                  = NAND_OEONTIME;

    nandTimingInfo->rdCycleTime               = NAND_RDCYCLETIME;
    nandTimingInfo->wrCycleTime               = NAND_WRCYCLETIME;
    nandTimingInfo->rdAccessTime              = NAND_RDACCESSTIME;
    nandTimingInfo->pageBurstAccessTime       = NAND_PAGEBURSTACCESSTIME;

    nandTimingInfo->cycle2CycleDelay          = NAND_CYCLE2CYCLEDELAY;
    nandTimingInfo->cycle2CycleDelaySameCSCfg = NAND_CYCLE2CYCLESAMECSEN;
    nandTimingInfo->cycle2CycleDelayDiffCSCfg = NAND_CYCLE2CYCLEDIFFCSEN;
    nandTimingInfo->busTAtime                 = NAND_BUSTURNAROUND;
}


/******************************************************************************
*                                                                             *
* \brief  Function to initialize the device and controller info.              *
*                                                                             *
* \param  nandInfo      : Pointer to structure containing controller and      *
*                         device information.                                 *
*                                                                             *
* \param  csNum         : Chip select where device is interfaced.             *
*                                                                             *
* \return none.                                                               *
*                                                                             *
******************************************************************************/
void BlPlatformNANDInfoInit(NandInfo_t *nandInfo)
{
    NandCtrlInfo_t *hNandCtrlInfo = nandInfo->hNandCtrlInfo;
    NandDmaInfo_t  *hNandDmaInfo  = nandInfo->hNandDmaInfo;
    NandEccInfo_t  *hNandEccInfo  = nandInfo->hNandEccInfo;

    /* Init the NAND Device Info */
    nandInfo->opMode                        = NAND_DATA_XFER_MODE;
    nandInfo->eccType                       = NAND_ECC_ALGO_BCH_8BIT;

    nandInfo->chipSelectCnt                 = 1;
    nandInfo->dieCnt                        = 1;
    nandInfo->chipSelects[0]                = NAND_CHIP_SELECT;
    nandInfo->busWidth                      = NAND_BUSWIDTH;
    nandInfo->pageSize                      = NAND_PAGE_SIZE_IN_BYTES;
    nandInfo->blkSize                       = NAND_BLOCK_SIZE_IN_BYTES;
    nandInfo->manId                         = NAND_MANUFATURER_MICRON_ID;
    nandInfo->devId                         = NAND_DEVICE_ID;
    nandInfo->dataRegAddr                   = (SOC_GPMC_0_REGS +
                                          GPMC_NAND_DATA(GPMC_CHIP_SELECT_0));
    nandInfo->addrRegAddr                   = (SOC_GPMC_0_REGS +
                                          GPMC_NAND_ADDRESS(GPMC_CHIP_SELECT_0));
    nandInfo->cmdRegAddr                    = (SOC_GPMC_0_REGS +
                                          GPMC_NAND_COMMAND(GPMC_CHIP_SELECT_0));
    /* Init the NAND Controller Info struct */
    hNandCtrlInfo->CtrlInit                 = GPMCNANDInit;
    hNandCtrlInfo->WaitPinStatusGet         = GPMCNANDWaitPinStatusGet;
    hNandCtrlInfo->currChipSelect           = NAND_CHIP_SELECT;
    hNandCtrlInfo->baseAddr                 = SOC_GPMC_0_REGS;
    hNandCtrlInfo->eccSupported             = (NAND_ECC_ALGO_HAMMING_1BIT |
                                          NAND_ECC_ALGO_BCH_4BIT |
                                          NAND_ECC_ALGO_BCH_8BIT |
                                          NAND_ECC_ALGO_BCH_16BIT );

    hNandCtrlInfo->waitPin                  = GPMC_WAIT_PIN0;
    hNandCtrlInfo->waitPinPol               = GPMC_WAIT_PIN_POLARITY_LOW;
    hNandCtrlInfo->wpPinPol                 = GPMC_WP_PIN_LEVEL_HIGH;
    hNandCtrlInfo->chipSelectBaseAddr[0]    = NAND_CS0_BASEADDR;
    hNandCtrlInfo->chipSelectRegionSize[0]  = NAND_CS0_REGIONSIZE;
    hNandCtrlInfo->hNandTimingInfo          = &nandTimingInfo;
    NANDTimingInfoInit(hNandCtrlInfo->hNandTimingInfo);


    /* Init the NAND Ecc Info */
    hNandEccInfo->baseAddr                  = SOC_ELM_0_REGS;
    hNandEccInfo->ECCInit                   = GPMCNANDECCInit;
    hNandEccInfo->ECCEnable                 = GPMCNANDECCEnable;
    hNandEccInfo->ECCDisable                = GPMCNANDECCDisable;
    hNandEccInfo->ECCWriteSet               = GPMCNANDECCWriteSet;
    hNandEccInfo->ECCReadSet                = GPMCNANDECCReadSet;
    hNandEccInfo->ECCCalculate              = GPMCNANDECCCalculate;
    hNandEccInfo->ECCCheckAndCorrect        = GPMCNANDECCCheckAndCorrect;

    /* Init the NAND DMA info */
    hNandDmaInfo->DMAXfer                   = NULL;
    hNandDmaInfo->DMAInit                   = NULL;
    hNandDmaInfo->DMAXferSetup              = NULL;
    hNandDmaInfo->DMAXferStatusGet          = NULL;
}

/*
 * \brief This function is used to initialize and configure NAND.
 *
 * \param  none.
 *
 * \return none
*/
void BlPlatformNANDSetup(void)
{
    /* Enable clock for NAND and Do the PINMUXing */
    NANDPinMuxSetup();
    GPMCClkConfig();
}
#endif

#if defined(MMCSD)
/*
 * \brief This function is used to initialize and configure NAND.
 *
 * \param  none.
 *
 * \return none
*/
void BlPlatformMMCSDSetup(void)
{
    /* Enable clock for MMCSD and Do the PINMUXing */
    HSMMCSDPinMuxSetup();
    HSMMCSDModuleClkConfig();
}
#endif


#if defined(SPI)
/*
 * \brief This function is used to initialize and configure SPI Module.
 *
 * \param  none.
 *
 * \return none
*/
void BlPlatformSPISetup(void)
{
    unsigned int regVal;

    /* Enable clock for SPI */
    regVal = (HWREG(SOC_CM_PER_REGS + CM_PER_SPI0_CLKCTRL) &
                    ~(CM_PER_SPI0_CLKCTRL_MODULEMODE));

    regVal |= CM_PER_SPI0_CLKCTRL_MODULEMODE_ENABLE;

    HWREG(SOC_CM_PER_REGS + CM_PER_SPI0_CLKCTRL) = regVal;

    /* Setup SPI PINMUX */
    McSPIPinMuxSetup(0);
    McSPI0CSPinMuxSetup(0);

    SPIConfigure();
}
#endif

/*
 * \brief This function is used to initialize and configure UART Module.
 *
 * \param  none.
 *
 * \return none
*/

void UARTSetup(void)
{
    volatile unsigned int regVal;

    /* Enable clock for UART0 */
    regVal = (HWREG(SOC_CM_WKUP_REGS + CM_WKUP_UART0_CLKCTRL) &
                    ~(CM_WKUP_UART0_CLKCTRL_MODULEMODE));

    regVal |= CM_WKUP_UART0_CLKCTRL_MODULEMODE_ENABLE;

    HWREG(SOC_CM_WKUP_REGS + CM_WKUP_UART0_CLKCTRL) = regVal;

    UARTStdioInit();
}

void initOppTable()
{
    oppTable[0].pllMult     = MPUPLL_M_275_MHZ ;  /* OPP100 275Mhz - 1.1v */
    oppTable[0].pmicVolt    = PMIC_VOLT_SEL_1100MV;  /* OPP100 275Mhz - 1.1v */
    oppTable[1].pllMult     = MPUPLL_M_500_MHZ ;  /* OPP100 500Mhz - 1.1v */
    oppTable[1].pmicVolt    = PMIC_VOLT_SEL_1100MV;  /* OPP100 500Mhz - 1.1v */
    oppTable[2].pllMult     = MPUPLL_M_600_MHZ ;  /* OPP120 600Mhz - 1.2v */
    oppTable[2].pmicVolt    = PMIC_VOLT_SEL_1200MV;  /* OPP120 600Mhz - 1.2v */
    oppTable[3].pllMult     = MPUPLL_M_720_MHZ ;  /* OPP TURBO 720Mhz - 1.26v */
    oppTable[3].pmicVolt    = PMIC_VOLT_SEL_1260MV;  /* OPP TURBO 720Mhz - 1.26v */
    oppTable[4].pllMult     = MPUPLL_M_300_MHZ ;  /* OPP50 300Mhz - 950mv */
    oppTable[4].pmicVolt    = PMIC_VOLT_SEL_0950MV;  /* OPP50 300Mhz - 950mv */
    oppTable[5].pllMult     = MPUPLL_M_300_MHZ ;  /* OPP100 300Mhz - 1.1v */
    oppTable[5].pmicVolt    = PMIC_VOLT_SEL_1100MV;  /* OPP100 300Mhz - 1.1v */
    oppTable[6].pllMult     = MPUPLL_M_600_MHZ ;  /* OPP100 600Mhz - 1.1v */
    oppTable[6].pmicVolt    = PMIC_VOLT_SEL_1100MV;  /* OPP100 600Mhz - 1.1v */
    oppTable[7].pllMult     = MPUPLL_M_720_MHZ ;  /* OPP120 720Mhz - 1.2v */
    oppTable[7].pmicVolt    = PMIC_VOLT_SEL_1200MV;  /* OPP120 720Mhz - 1.2v */
    oppTable[8].pllMult     = MPUPLL_M_800_MHZ ;  /* OPP TURBO 800Mhz - 1.26v */
    oppTable[8].pmicVolt    = PMIC_VOLT_SEL_1260MV;  /* OPP TURBO 800Mhz - 1.26v */
    oppTable[9].pllMult     = MPUPLL_M_1000_MHZ;  /* OPP NITRO 1000Mhz - 1.325v */
    oppTable[9].pmicVolt    = PMIC_VOLT_SEL_1325MV;  /* OPP NITRO 1000Mhz - 1.325v */
}

/*
 * \brief This function Initializes Pll, DDR and Uart
 *
 * \param  none
 *
 * \return none
*/
void BlPlatformConfig(void)
{
	UTILsICSSPRCMEnableStep1();

#ifdef FASTBOOT_PHY
	Ethernet_PHY_SYSBOOT_Mux();
	Ethernet_PHY_Reset();
#endif //FASTBOOT_PHY

	initOppTable();
    BoardInfoInit();
    deviceVersion = DeviceVersionGet();
    ConfigVddOpVoltage();

    oppMaxIdx = BootMaxOppGet();
    
    #if (defined HW_ICE) || (defined HW_ICE_V2) || (defined HW_IDK)
    oppMaxIdx = 6;
    #endif

    SetVdd1OpVoltage(oppTable[oppMaxIdx].pmicVolt);

    HWREG(SOC_WDT_1_REGS + WDT_WSPR) = 0xAAAAu;
    while(HWREG(SOC_WDT_1_REGS + WDT_WWPS) != 0x00);

    HWREG(SOC_WDT_1_REGS + WDT_WSPR) = 0x5555u;
    while(HWREG(SOC_WDT_1_REGS + WDT_WWPS) != 0x00);

    /* Configure DDR frequency */
#if defined (evmskAM335x) || defined (HW_ICE_V2)
    freqMultDDR = DDRPLL_M_DDR3;
#elif evmAM335x
    if(BOARD_ID_EVM_DDR3 == BoardIdGet())
    {
        freqMultDDR = DDRPLL_M_DDR3;
    }
    else if(BOARD_ID_EVM_DDR2 == BoardIdGet())
    {
        freqMultDDR = DDRPLL_M_DDR2;
    }
    else
    	freqMultDDR = DDRPLL_M_DDR2;
#else
    freqMultDDR = DDRPLL_M_DDR2;
#endif

    /* Set the PLL0 to generate 300MHz for ARM */
    PLLInit();

    /* Enable the control module */
    HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CONTROL_CLKCTRL) =
            CM_WKUP_CONTROL_CLKCTRL_MODULEMODE_ENABLE;

    /* EMIF Initialization */
    EMIFInit();

    /* DDR Initialization */

#if defined (evmskAM335x) || defined (HW_ICE_V2)
    /* Enable DDR_VTT */
    DDRVTTEnable();
    DDR3Init();
#elif evmAM335x
    if(BOARD_ID_EVM_DDR3 == BoardIdGet())
    {
        DDR3Init();
    }
    else if(BOARD_ID_EVM_DDR2 == BoardIdGet())
    {
        DDR2Init();
    }
    else
    	DDR2Init();
#else
    DDR2Init();
#endif



    /* UART Initialization */
    //UARTSetup();
}

/*
** Enable DDR_VTT.
**
*/
#if defined (evmskAM335x) || defined (HW_ICE_V2)

/**
 * \brief  This function does the pin multiplexing for any GPIO Pin.
 *
 * \param  offsetAddr   This is the offset address of the Pad Control Register
 *                      corresponding to the GPIO pin. These addresses are
 *                      offsets with respect to the base address of the
 *                      Control Module.
 * \param  padConfValue This is the value to be written to the Pad Control
 *                      register whose offset address is given by 'offsetAddr'.
 *
 * The 'offsetAddr' and 'padConfValue' can be obtained from macros defined
 * in the file 'include/armv7a/am335x/pin_mux.h'.\n
 *
 * \return  None.
 */

static void DDRVTTEnable(void)
{

    GPIO0ModuleClkConfig();

#ifdef HW_ICE_V2
    GPIO_PMUX_OFFADDR_VALUE(0, 18, PAD_FS_RXE_PD_PUPDE(7));
#else
    GPIO_PMUX_OFFADDR_VALUE(0, 7, PAD_FS_RXE_PD_PUPDE(7));
#endif

    /* Resetting the GPIO module. */
    GPIOModuleReset(SOC_GPIO_0_REGS);

    /* Enabling the GPIO module. */
    GPIOModuleEnable(SOC_GPIO_0_REGS);

    /* Setting the GPIO pin as an input pin. */    
    GPIODirModeSet(SOC_GPIO_0_REGS,
                   GPIO_INSTANCE_PIN_NUMBER,
                   GPIO_DIR_OUTPUT);

    GPIOPinWrite(SOC_GPIO_0_REGS, GPIO_INSTANCE_PIN_NUMBER,
                 GPIO_PIN_HIGH);

}
#endif

/*
 * \brief This function does any post boot setup/init
 *
 * \param  none
 *
 * \return none
*/
void BlPlatformConfigPostBoot( void )
{
}

/*
 * \brief This function copies the image form SPI to RAM
 *
 * \param  none
 *
 * \return Status of the copy operation.
*/
#if defined(SPI)

int BlPlatformSPIImageCopy()
{
    ti_header imageHdr = {0};
#ifdef MEASURE_BOOT_TIME
        GPIOPinWrite(BOOTTIME_GPIO_BASE_ADDR,BOOTTIME_GPIO_NUM,GPIO_PIN_HIGH);
#endif
    /* Spi read function to read the header */
    BlSPIReadFlash(IMAGE_OFFSET, sizeof(ti_header), (unsigned char *)&imageHdr);

	if ((imageHdr.magic_number != MAGIC_NUM_SPI) ||
	    (imageHdr.image_size == 0))
    {
#ifdef MEASURE_BOOT_TIME
        GPIOPinWrite(BOOTTIME_GPIO_BASE_ADDR,BOOTTIME_GPIO_NUM,GPIO_PIN_LOW);
#endif     
		return E_FAIL;
	}
    BlSPIReadFlash(IMAGE_OFFSET + sizeof(ti_header), imageHdr.image_size, (unsigned char *)imageHdr.load_addr);
    entryPoint = imageHdr.run_addr;
	return E_PASS;
}
#endif

/*
 * \brief This function copies the image form MMCSD to RAM
 *
 * \param  none
 *
 * \return Status of the copy operation.
*/

#if defined(MMCSD)

int BlPlatformMMCSDImageCopy()
{
    if(HSMMCSDInit() == E_FAIL)
        return E_FAIL;
    return HSMMCSDImageCopy();
}

#endif

/*
 * \brief This function copies the image form NAND to RAM
 *
 * \param  none
 *
 * \return Status of the copy operation.
*/

#if defined(NAND)

unsigned int BlPlatformNANDImageCopy(NandInfo_t *nandInfo)
{

    ti_header header;

    /* Spi read function to read the header */
    BlNANDReadFlash(nandInfo, IMAGE_OFFSET, sizeof(header), (unsigned char *)&header);

    /* Copies application from SPI flash to DDR RAM */
    BlNANDReadFlash( nandInfo, IMAGE_OFFSET + sizeof(header),
                     header.image_size, (unsigned char *)(header.load_addr) );

    entryPoint = (unsigned int)header.load_addr;

    return (TRUE);
}

#endif


#ifdef FASTBOOT_CONFIGNOR
void BlPlatformNORInit(void)
{
    unsigned int temp;

    HWREG( SOC_PRCM_REGS + CM_PER_GPIO2_CLKCTRL ) |= CM_PER_GPIO2_CLKCTRL_MODULEMODE_ENABLE;
    HWREG( SOC_PRCM_REGS + CM_PER_GPIO3_CLKCTRL ) |= CM_PER_GPIO3_CLKCTRL_MODULEMODE_ENABLE;

    /* pinmuxing */
    GPMCPinMuxConfNor();

    /* GPIO3_4 controlls SYSBOOT latch */
    /* Disable the external buffers using GPIO3_4 to discard the SYSBOOT Outputs */
    HWREG(SOC_GPIO_3_REGS + 0x134) &= ~(1<<4);
    HWREG(SOC_GPIO_3_REGS + 0x194) = (1<<4);
    HWREG(SOC_GPIO_2_REGS + 0x134) &=~((1<<12) | (1<<13));
    HWREG(SOC_GPIO_2_REGS + 0x190) = (1<<12) | (1<<13);


	//enable clock to GPMC module
    HWREG(SOC_PRCM_REGS + CM_PER_GPMC_CLKCTRL ) |=
        		CM_PER_GPMC_CLKCTRL_MODULEMODE_ENABLE;
    //check to see if enabled
    while( (HWREG(SOC_PRCM_REGS + CM_PER_GPMC_CLKCTRL) & CM_PER_GPMC_CLKCTRL_IDLEST) !=
        		(CM_PER_GPMC_CLKCTRL_IDLEST_FUNC << CM_PER_GPMC_CLKCTRL_IDLEST_SHIFT));

    //reset the GPMC module
    HWREG(GPMC_BASE + GPMC_SYSCONFIG ) |= GPMC_SYSCONFIG_SOFTRESET;
    while((HWREG(GPMC_BASE + GPMC_SYSSTATUS) & GPMC_SYSSTATUS_RESETDONE) ==
        		GPMC_SYSSTATUS_RESETDONE_RSTONGOING);

    //Configure to no idle
    temp = HWREG(GPMC_BASE + GPMC_SYSCONFIG);
    temp &= ~GPMC_SYSCONFIG_IDLEMODE;
    temp |= GPMC_SYSCONFIG_IDLEMODE_NOIDLE << GPMC_SYSCONFIG_IDLEMODE_SHIFT;
    HWREG(GPMC_BASE + GPMC_SYSCONFIG) = temp;

    HWREG(GPMC_BASE + GPMC_IRQENABLE) = 0x0;
    HWREG(GPMC_BASE + GPMC_TIMEOUT_CONTROL) = 0x0;

    /* Memory type configuration */
    HWREG(GPMC_BASE + GPMC_CONFIG1(GPMC_CHIP_SELECT_0)) = (0x0 |
    	 (GPMC_CONFIG1_0_DEVICESIZE_SIXTEENBITS << GPMC_CONFIG1_0_DEVICESIZE_SHIFT ) |
	     (GPMC_CONFIG1_0_ATTACHEDDEVICEPAGELENGTH_FOUR << GPMC_CONFIG1_0_ATTACHEDDEVICEPAGELENGTH_SHIFT ) |
	     (GPMC_CONFIG1_0_MUXADDDATA_MUX << GPMC_CONFIG1_0_MUXADDDATA_SHIFT ));

    /* Chip select timing configuration */
    HWREG(GPMC_BASE + GPMC_CONFIG2(GPMC_CHIP_SELECT_0)) = (0x0 |
    	 (CS_ON_TIME << GPMC_CONFIG2_0_CSONTIME_SHIFT) |
         (CS_DEASSERT_RD << GPMC_CONFIG2_0_CSRDOFFTIME_SHIFT) |
         (CS_DEASSERT_WR << GPMC_CONFIG2_0_CSWROFFTIME_SHIFT));

    /* ADV# signal timing configuration */
    HWREG(GPMC_BASE + GPMC_CONFIG3(GPMC_CHIP_SELECT_0)) = (0x0 |
         (ADV_ASSERT << GPMC_CONFIG3_0_ADVONTIME_SHIFT) |
	     (ADV_DEASSERT_RD << GPMC_CONFIG3_0_ADVRDOFFTIME_SHIFT) |
	     (ADV_DEASSERT_WR << GPMC_CONFIG3_0_ADVWROFFTIME_SHIFT));

    /* WE# and OE# signal timing configuration */
    HWREG(GPMC_BASE + GPMC_CONFIG4(GPMC_CHIP_SELECT_0)) = (0x0 |
         (OE_ASSERT << GPMC_CONFIG4_0_OEONTIME_SHIFT) |
	     (OE_DEASSERT << GPMC_CONFIG4_0_OEOFFTIME_SHIFT) |
	     (WE_ASSERT << GPMC_CONFIG4_0_WEONTIME_SHIFT) |
	     (WE_DEASSERT << GPMC_CONFIG4_0_WEOFFTIME_SHIFT));

    /* RdAccessTime and CycleTime timing configuration */
    HWREG(GPMC_BASE + GPMC_CONFIG5(GPMC_CHIP_SELECT_0)) = (0x0 |
	     (RD_CYCLE_TIM << GPMC_CONFIG5_0_RDCYCLETIME_SHIFT) |
	     (WR_CYCLE_TIM << GPMC_CONFIG5_0_WRCYCLETIME_SHIFT) |
	     (RD_ACCESS_TIM << GPMC_CONFIG5_0_RDACCESSTIME_SHIFT));

    /* WrAccessTime, WrDataOnADmuxBus, Cycle2Cycle and BusTurnAround timing configuration */
    HWREG(GPMC_BASE + GPMC_CONFIG6(GPMC_CHIP_SELECT_0)) = (0x0 |
		 (GPMC_CONFIG6_0_CYCLE2CYCLESAMECSEN_C2CDELAY << GPMC_CONFIG6_0_CYCLE2CYCLESAMECSEN_SHIFT) |
		 (CYC2CYC_DELAY << GPMC_CONFIG6_0_CYCLE2CYCLEDELAY_SHIFT) |
	     (WR_DATA_ON_ADMUX << GPMC_CONFIG6_0_WRDATAONADMUXBUS_SHIFT) |
	     (WR_ACCESS_TIM << GPMC_CONFIG6_0_WRACCESSTIME_SHIFT));

    /* Chip-select address mapping configuration */
    HWREG(GPMC_BASE + GPMC_CONFIG7(GPMC_CHIP_SELECT_0)) =
         (NOR_BASE_ADDR << GPMC_CONFIG7_0_BASEADDRESS_SHIFT) |
         (GPMC_CONFIG7_0_CSVALID_CSENABLED << GPMC_CONFIG7_0_CSVALID_SHIFT) |
         (NOR_MASK << GPMC_CONFIG7_0_MASKADDRESS_SHIFT);
}

void GPMCPinMuxConfNor(void)
{
	HWREG(CTRL_MODULE_BASE_ADDR + CONTROL_CONF_GPMC_AD(0)) =
				  PINMUXMODE_0 |
					 CONTROL_CONF_GPMC_AD0_CONF_GPMC_AD0_RXACTIVE |
					 CONTROL_CONF_GPMC_AD0_CONF_GPMC_AD0_PUDEN;

	HWREG(CTRL_MODULE_BASE_ADDR + CONTROL_CONF_GPMC_AD(1)) =
				  PINMUXMODE_0 |
					 CONTROL_CONF_GPMC_AD1_CONF_GPMC_AD1_RXACTIVE |
					 CONTROL_CONF_GPMC_AD1_CONF_GPMC_AD1_PUDEN;

	HWREG(CTRL_MODULE_BASE_ADDR + CONTROL_CONF_GPMC_AD(2)) =
				  PINMUXMODE_0 |
					 CONTROL_CONF_GPMC_AD2_CONF_GPMC_AD2_RXACTIVE |
					 CONTROL_CONF_GPMC_AD2_CONF_GPMC_AD2_PUDEN;

	HWREG(CTRL_MODULE_BASE_ADDR + CONTROL_CONF_GPMC_AD(3)) =
				  PINMUXMODE_0 |
					 CONTROL_CONF_GPMC_AD3_CONF_GPMC_AD3_RXACTIVE |
					 CONTROL_CONF_GPMC_AD3_CONF_GPMC_AD3_PUDEN;

	HWREG(CTRL_MODULE_BASE_ADDR + CONTROL_CONF_GPMC_AD(4)) =
				  PINMUXMODE_0 |
					 CONTROL_CONF_GPMC_AD4_CONF_GPMC_AD4_RXACTIVE |
					 CONTROL_CONF_GPMC_AD4_CONF_GPMC_AD4_PUDEN;

	HWREG(CTRL_MODULE_BASE_ADDR + CONTROL_CONF_GPMC_AD(5)) =
				  PINMUXMODE_0 |
					 CONTROL_CONF_GPMC_AD5_CONF_GPMC_AD5_RXACTIVE |
					 CONTROL_CONF_GPMC_AD5_CONF_GPMC_AD5_PUDEN;

	HWREG(CTRL_MODULE_BASE_ADDR + CONTROL_CONF_GPMC_AD(6)) =
				  PINMUXMODE_0 |
					 CONTROL_CONF_GPMC_AD6_CONF_GPMC_AD6_RXACTIVE |
					 CONTROL_CONF_GPMC_AD6_CONF_GPMC_AD6_PUDEN;

	HWREG(CTRL_MODULE_BASE_ADDR + CONTROL_CONF_GPMC_AD(7)) =
				  PINMUXMODE_0 |
					 CONTROL_CONF_GPMC_AD7_CONF_GPMC_AD7_RXACTIVE |
					 CONTROL_CONF_GPMC_AD7_CONF_GPMC_AD7_PUDEN;

	HWREG(CTRL_MODULE_BASE_ADDR + CONTROL_CONF_GPMC_AD(8)) =
				  PINMUXMODE_0 |
					 CONTROL_CONF_GPMC_AD8_CONF_GPMC_AD8_RXACTIVE |
					 CONTROL_CONF_GPMC_AD8_CONF_GPMC_AD8_PUDEN;

	HWREG(CTRL_MODULE_BASE_ADDR + CONTROL_CONF_GPMC_AD(9)) =
				  PINMUXMODE_0 |
					 CONTROL_CONF_GPMC_AD9_CONF_GPMC_AD9_RXACTIVE |
					 CONTROL_CONF_GPMC_AD9_CONF_GPMC_AD9_PUDEN;

	HWREG(CTRL_MODULE_BASE_ADDR + CONTROL_CONF_GPMC_AD(10)) =
				  PINMUXMODE_0 |
					 CONTROL_CONF_GPMC_AD10_CONF_GPMC_AD10_RXACTIVE |
					 CONTROL_CONF_GPMC_AD10_CONF_GPMC_AD10_PUDEN;

	HWREG(CTRL_MODULE_BASE_ADDR + CONTROL_CONF_GPMC_AD(11)) =
				  PINMUXMODE_0 |
					 CONTROL_CONF_GPMC_AD11_CONF_GPMC_AD11_RXACTIVE |
					 CONTROL_CONF_GPMC_AD11_CONF_GPMC_AD11_PUDEN;

	HWREG(CTRL_MODULE_BASE_ADDR + CONTROL_CONF_GPMC_AD(12)) =
				  PINMUXMODE_0 |
					 CONTROL_CONF_GPMC_AD12_CONF_GPMC_AD12_RXACTIVE |
					 CONTROL_CONF_GPMC_AD12_CONF_GPMC_AD12_PUDEN;

	HWREG(CTRL_MODULE_BASE_ADDR + CONTROL_CONF_GPMC_AD(13)) =
				  PINMUXMODE_0 |
					 CONTROL_CONF_GPMC_AD13_CONF_GPMC_AD13_RXACTIVE |
					 CONTROL_CONF_GPMC_AD13_CONF_GPMC_AD13_PUDEN;

	HWREG(CTRL_MODULE_BASE_ADDR + CONTROL_CONF_GPMC_AD(14)) =
				  PINMUXMODE_0 |
					 CONTROL_CONF_GPMC_AD14_CONF_GPMC_AD14_RXACTIVE |
					 CONTROL_CONF_GPMC_AD14_CONF_GPMC_AD14_PUDEN;

	HWREG(CTRL_MODULE_BASE_ADDR + CONTROL_CONF_GPMC_AD(15)) =
				  PINMUXMODE_0 |
					 CONTROL_CONF_GPMC_AD15_CONF_GPMC_AD15_RXACTIVE |
					 CONTROL_CONF_GPMC_AD15_CONF_GPMC_AD15_PUDEN;


	HWREG(CTRL_MODULE_BASE_ADDR + CONTROL_CONF_LCD_VSYNC) =           //GPMC_A1 --> A16
					PINMUXMODE_2 |
					  CONTROL_CONF_LCD_VSYNC_CONF_LCD_VSYNC_RXACTIVE |
					  CONTROL_CONF_LCD_VSYNC_CONF_LCD_VSYNC_PUDEN;

	HWREG(CTRL_MODULE_BASE_ADDR + CONTROL_CONF_LCD_HSYNC) =          //GPMC_A2 --> A17
					PINMUXMODE_2 |
						  CONTROL_CONF_LCD_HSYNC_CONF_LCD_HSYNC_RXACTIVE |
							 CONTROL_CONF_LCD_HSYNC_CONF_LCD_HSYNC_PUDEN;

	HWREG(CTRL_MODULE_BASE_ADDR + CONTROL_CONF_LCD_DATA(6)) =			//A18 --> GPIO2_12
					PINMUXMODE_7 |
						  CONTROL_CONF_LCD_DATA6_CONF_LCD_DATA6_RXACTIVE |
							 CONTROL_CONF_LCD_DATA6_CONF_LCD_DATA6_PUDEN;

	HWREG(CTRL_MODULE_BASE_ADDR + CONTROL_CONF_LCD_DATA(7)) =         //A19 --> GPIO2_13
					PINMUXMODE_7 |
						  CONTROL_CONF_LCD_DATA7_CONF_LCD_DATA7_RXACTIVE |
							 CONTROL_CONF_LCD_DATA7_CONF_LCD_DATA7_PUDEN;
	HWREG(CTRL_MODULE_BASE_ADDR + CONTROL_CONF_MII1_RXDV) =         //GPIO3_4 to disable SYSBOOT buffers
							PINMUXMODE_7 |
								  CONTROL_CONF_LCD_DATA7_CONF_LCD_DATA7_RXACTIVE |
									 CONTROL_CONF_LCD_DATA7_CONF_LCD_DATA7_PUDEN;

   /* control pins */
    HWREG(CTRL_MODULE_BASE_ADDR + CONTROL_CONF_GPMC_WEN ) =
   	      PINMUXMODE_0 |
   	       CONTROL_CONF_GPMC_OEN_REN_CONF_GPMC_OEN_REN_RXACTIVE |
           CONTROL_CONF_GPMC_OEN_REN_CONF_GPMC_OEN_REN_PUDEN;

    HWREG(CTRL_MODULE_BASE_ADDR + CONTROL_CONF_GPMC_CSN(0) ) =
   	      PINMUXMODE_0 |
          CONTROL_CONF_GPMC_OEN_REN_CONF_GPMC_OEN_REN_RXACTIVE |
          CONTROL_CONF_GPMC_OEN_REN_CONF_GPMC_OEN_REN_PUDEN;

    HWREG(CTRL_MODULE_BASE_ADDR + CONTROL_CONF_GPMC_OEN_REN) =
     	      PINMUXMODE_0 |
              CONTROL_CONF_GPMC_OEN_REN_CONF_GPMC_OEN_REN_RXACTIVE |
              CONTROL_CONF_GPMC_OEN_REN_CONF_GPMC_OEN_REN_PUDEN;

    HWREG(CTRL_MODULE_BASE_ADDR + CONTROL_CONF_GPMC_ADVN_ALE ) =
    	( PINMUXMODE_0 |
    	CONTROL_CONF_GPMC_ADVN_ALE_CONF_GPMC_ADVN_ALE_RXACTIVE |
    	CONTROL_CONF_GPMC_ADVN_ALE_CONF_GPMC_ADVN_ALE_PUDEN );

    return;
}
#endif

#define NOR_SCTR_SIZE (0x10000)
#define NOR_1ST_SCTR_ADDR (0x08000000)
#define NOR_2ND_SCTR_ADDR (0x08010000)

#ifdef FASTBOOT_SPINOR
int find_NORImage_from_SPI_exec(void)
{
	unsigned int t_addr = 0;
#ifndef HW_ICE_V2
	unsigned int t_blk = 0;
#endif
	unsigned int sector;
	void (*appEntryNOR)();
	ti_header t_header;

    //pinmux_NOR();

	t_addr = NOR_1ST_SCTR_ADDR;

	for(sector = 1 ; sector < 10 ; sector++)
	{
	#ifdef HW_ICE_V2
		t_addr = NOR_1ST_SCTR_ADDR+ (sector * NOR_SCTR_SIZE);
	#else
		t_addr = ((sector%2)?NOR_2ND_SCTR_ADDR:NOR_1ST_SCTR_ADDR);
		t_blk = (sector/2);
		//Find GPIO Pin value
		t_blk <<= 26;
		//Set GPIO Pin value
		*(( int*) 0x481AC13C) = t_blk;
	#endif
		t_header.magic_number = ((ti_header*)t_addr)->magic_number;
		t_header.image_size = ((ti_header*)t_addr)->image_size;
		t_header.load_addr = ((ti_header*)t_addr)->load_addr;
		t_header.run_addr = ((ti_header*)t_addr)->run_addr;


		if((t_header.magic_number == MAGIC_NUM_NOR) &&
		   (t_header.image_size != 0))
		{
			appEntryNOR = (void (*)(void)) t_header.run_addr;
			(*appEntryNOR)( );
		}
 	}
	#ifndef HW_ICE_V2
	*(( int*) 0x481AC13C) = 0;
	#endif
	return -1;
}
#endif

#ifdef FASTBOOT_PHY

//t_mdio_params mdio_conf = {0x9F, 0x1, 0x3, 1, 1, 1};	// default configuration for ICEv2
tmdio_params mdio_conf = {12, 0x1, 0x3, 1, 1, 1};	// default configuration for ICEv2
tphy_initdata phy_initdata[] = PHY_INITDATA_TLK110_FIXED_CONFIG;

void mdio_phy_initconfig (tphy_initdata *initdata)
{
	unsigned int temp32;
	while(initdata->phyMdioAddr != -1)
	{
		if(initdata->phyRegValue != -1)
		{
			MDIO_write(initdata->mdio_useraccess, initdata->phyMdioAddr, initdata->phyRegAddr, initdata->phyRegValue);
		}else
		{
			temp32 = MDIO_read(initdata->mdio_useraccess, initdata->phyMdioAddr, initdata->phyRegAddr);
			temp32 &= ~(initdata->phyRegClrValue);
			temp32 |= initdata->phyRegSetValue;
			MDIO_write(initdata->mdio_useraccess, initdata->phyMdioAddr, initdata->phyRegAddr, temp32);
		}
		initdata++;
	};

}

void mdio_phy_config (void)
{
	unsigned int temp32, RegAddr;
	unsigned short temp16;

	HWREG(CTRL_MODULE_BASE_ADDR + CONTROL_CONF_GPMC_CSN(3)) = 0 |
				  PINMUXMODE_5 |
				  CONTROL_CONF_GPMC_CSN3_CONF_GPMC_CSN3_RXACTIVE |
				  CONTROL_CONF_GPMC_CSN3_CONF_GPMC_CSN3_PUDEN;

	HWREG(CTRL_MODULE_BASE_ADDR + CONTROL_CONF_GPMC_CLK) = 0 |
				  PINMUXMODE_5 |
				  CONTROL_CONF_GPMC_CLK_CONF_GPMC_CLK_RXACTIVE |
				  CONTROL_CONF_GPMC_CLK_CONF_GPMC_CLK_PUDEN;

	/* Optain Ethernet PHY settings from memory - this is the last known good configuration */
	/* Need to avoid auto MDI/MDX and auto-negotiation in order to get a Ehthernet link fast */

	HWREG(PRUSS_MDIO_BASE + PRU_MDIO_CNTL) = 0x40000000 | mdio_conf.clkdiv; //200/(clkdiv+1)

	GPIOPinWrite(BOOTTIME_GPIO_BASE_ADDR,BOOTTIME_GPIO_NUM,GPIO_PIN_HIGH);

	/* wait for the PHYs to be detected via MDIO */
	temp32 = (1<<mdio_conf.addr0) | (1<<mdio_conf.addr1);
	do{
		;
		//temp32 = HWREG(PRUSS_MDIO_BASE + PRU_MDIO_ALIVE);
		//temp32 &= (1<<mdio_conf.addr0) | (1<<mdio_conf.addr1);
	}while(temp32 != HWREG(PRUSS_MDIO_BASE + PRU_MDIO_ALIVE));

	GPIOPinWrite(BOOTTIME_GPIO_BASE_ADDR,BOOTTIME_GPIO_NUM,GPIO_PIN_LOW);

    mdio_phy_initconfig(phy_initdata);

}
#endif

void UTILsICSSPRCMEnableStep1(void)
{
	HWREG( SOC_PRM_PER_REGS)      |= 0x2;
	HWREG( SOC_PRM_PER_REGS)      &= 0xFFFFFFFD;
	HWREG( SOC_PRCM_REGS + CM_PER_ICSS_CLKCTRL)      |= 0x2;
}

void UTILsICSSPRCMReset(void)
{
	HWREG( SOC_PRM_PER_REGS)   |= 0x2;
	HWREG( SOC_PRM_PER_REGS)   &= 0xFFFFFFFD;
}

#ifdef FASTBOOT_PHY
void Ethernet_PHY_SYSBOOT_Mux(void)
{
	// power up gpio3
    HWREG( SOC_PRCM_REGS + CM_PER_GPIO3_CLKCTRL ) |= CM_PER_GPIO3_CLKCTRL_MODULEMODE_ENABLE;
    GPIOModuleEnable(SOC_GPIO_3_REGS);
    //Set pin direction for gpio3_10 (MUX_MII_CTRL1) & gpio3_4 (PR1_MII_CTRL)
    HWREG(SOC_GPIO_3_REGS + 0x134) &= ~((1<<10) | (1<<4));
    //Set pin
    HWREG(SOC_GPIO_3_REGS + 0x194) = (1<<4);	// PR1_MII_CTRL -> '1'
    HWREG(SOC_GPIO_3_REGS + 0x190) = (1<<10);	// MUX_MII_CTRL1 -> '0'

	// PHY0
    // TLK_RXD0
    HWREG(CTRL_MODULE_BASE_ADDR + CONTROL_CONF_LCD_DATA(11)) = 0 |
				  PINMUXMODE_5 |
				  CONTROL_CONF_LCD_DATA11_CONF_LCD_DATA11_RXACTIVE |
				  CONTROL_CONF_LCD_DATA11_CONF_LCD_DATA11_PUDEN;
    HWREG(CTRL_MODULE_BASE_ADDR + CONTROL_CONF_MII1_RXD0) = 0 |
				  PINMUXMODE_5 |
				  CONTROL_CONF_MII1_RXD0_CONF_MII1_RXD0_RXACTIVE |
				  CONTROL_CONF_MII1_RXD0_CONF_MII1_RXD0_PUDEN;
    // TLK_RXD1
    HWREG(CTRL_MODULE_BASE_ADDR + CONTROL_CONF_LCD_DATA(10)) = 0 |
				  PINMUXMODE_5 |
				  CONTROL_CONF_LCD_DATA10_CONF_LCD_DATA10_RXACTIVE |
				  CONTROL_CONF_LCD_DATA10_CONF_LCD_DATA10_PUDEN;
    HWREG(CTRL_MODULE_BASE_ADDR + CONTROL_CONF_MII1_RXD1) = 0 |
				  PINMUXMODE_5 |
				  CONTROL_CONF_MII1_RXD1_CONF_MII1_RXD1_RXACTIVE |
				  CONTROL_CONF_MII1_RXD1_CONF_MII1_RXD1_PUDEN;
    // TLK_RXD2
    HWREG(CTRL_MODULE_BASE_ADDR + CONTROL_CONF_LCD_DATA(9)) = 0 |
				  PINMUXMODE_5 |
				  CONTROL_CONF_LCD_DATA9_CONF_LCD_DATA9_RXACTIVE |
				  CONTROL_CONF_LCD_DATA9_CONF_LCD_DATA9_PUDEN;
    // TLK_RXD3
    HWREG(CTRL_MODULE_BASE_ADDR + CONTROL_CONF_LCD_DATA(8)) = 0 |
				  PINMUXMODE_5 |
				  CONTROL_CONF_LCD_DATA8_CONF_LCD_DATA8_RXACTIVE |
				  CONTROL_CONF_LCD_DATA8_CONF_LCD_DATA8_PUDEN;


	// PHY1
    // TLK_COL
    HWREG(CTRL_MODULE_BASE_ADDR + CONTROL_CONF_GPMC_WAIT0) = 0 |
				  PINMUXMODE_5 |
				  CONTROL_CONF_GPMC_WAIT0_CONF_GPMC_WAIT0_RXACTIVE |
				  CONTROL_CONF_GPMC_WAIT0_CONF_GPMC_WAIT0_PUDEN;
    // TLK_RXD0
    HWREG(CTRL_MODULE_BASE_ADDR + CONTROL_CONF_GPMC_A(8)) = 0 |
				  PINMUXMODE_5 |
				  CONTROL_CONF_GPMC_A8_CONF_GPMC_A8_RXACTIVE |
				  CONTROL_CONF_GPMC_A8_CONF_GPMC_A8_PUDEN;
    // TLK_RXD1
    HWREG(CTRL_MODULE_BASE_ADDR + CONTROL_CONF_GPMC_A(7)) = 0 |
				  PINMUXMODE_5 |
				  CONTROL_CONF_GPMC_A7_CONF_GPMC_A7_RXACTIVE |
				  CONTROL_CONF_GPMC_A7_CONF_GPMC_A7_PUDEN;
    // TLK_RXD2
    HWREG(CTRL_MODULE_BASE_ADDR + CONTROL_CONF_GPMC_A(6)) = 0 |
				  PINMUXMODE_5 |
				  CONTROL_CONF_GPMC_A6_CONF_GPMC_A6_RXACTIVE |
				  CONTROL_CONF_GPMC_A6_CONF_GPMC_A6_PUDEN;
    // TLK_RXD3
    HWREG(CTRL_MODULE_BASE_ADDR + CONTROL_CONF_GPMC_A(5)) = 0 |
				  PINMUXMODE_5 |
				  CONTROL_CONF_GPMC_A5_CONF_GPMC_A5_RXACTIVE |
				  CONTROL_CONF_GPMC_A5_CONF_GPMC_A5_PUDEN;


}

void Ethernet_PHY_Reset(void)
{
	unsigned int i;

	//GPIOPinWrite(BOOTTIME_GPIO_BASE_ADDR,BOOTTIME_GPIO_NUM,GPIO_PIN_LOW);

	// power up gpio2
	HWREG( SOC_PRCM_REGS + CM_PER_GPIO2_CLKCTRL ) |= CM_PER_GPIO2_CLKCTRL_MODULEMODE_ENABLE;
    GPIOModuleEnable(SOC_GPIO_2_REGS);
    //Set pin direction for gpio2_5 (PHY reset)
    HWREG(SOC_GPIO_2_REGS + 0x134) &= ~(1<<5);
    HWREG(SOC_GPIO_2_REGS + 0x190) = (1<<5);
    // assert Reset for >=1usec
    for(i=0; i<=400; i++)
    {
    	;
    }
    HWREG(SOC_GPIO_2_REGS + 0x194) = (1<<5);

    //GPIOPinWrite(BOOTTIME_GPIO_BASE_ADDR,BOOTTIME_GPIO_NUM,GPIO_PIN_HIGH);
}

void Ethernet_PHY_Register_Dump(void)
{
	unsigned int temp32, RegAddr;
	temp32 = 0x4a300000; //PRU0 data memory
	for(RegAddr=0; RegAddr<=0x1F; RegAddr++)
	{
		HWREG(temp32) = MDIO_read(0, mdio_conf.addr0, RegAddr);
		HWREG(temp32+0x100) = MDIO_read(1, mdio_conf.addr1, RegAddr);
		temp32 += 4;
	}

#if 0
	unsigned int RegAddr, UserAccess, temp32[0x20];

	for(RegAddr=0; RegAddr<=0x1F; RegAddr++)
	{
		temp32[RegAddr] = MDIO_read(0, mdio_conf.addr0, RegAddr);
	}

	RegAddr = 0;
#endif
}


unsigned short MDIO_read (unsigned char usraccess_channel, unsigned char PhyAddr, unsigned short RegAddr)
{
	unsigned int UserAccess, pruss_mdio_addr = PRUSS_MDIO_BASE;

	if(usraccess_channel == 0)
		pruss_mdio_addr += PRU_MDIO_USRACCESS0;
	else
		pruss_mdio_addr += PRU_MDIO_USRACCESS1;

	while(HWREG(pruss_mdio_addr)&0x80000000)
	{
		;
	}

	if(RegAddr<=0xFF)
	{
		UserAccess = (1<<31) | (0<<30) | (0<<29) | (RegAddr<<21) | (PhyAddr<<16);
		HWREG(pruss_mdio_addr) = UserAccess;
		while(UserAccess & 0x80000000)
			UserAccess = HWREG(pruss_mdio_addr);
	}else
	{
		return (0xFFFF);
	}
	return(UserAccess & 0xFFFF);
}

unsigned short MDIO_write (unsigned char usraccess_channel, unsigned char PhyAddr, unsigned short RegAddr, unsigned short RegValue)
{
	unsigned int UserAccess, pruss_mdio_addr = PRUSS_MDIO_BASE;

	if(usraccess_channel == 0)
		pruss_mdio_addr += PRU_MDIO_USRACCESS0;
	else
		pruss_mdio_addr += PRU_MDIO_USRACCESS1;

	if(RegAddr<=0xFF)
	{
		UserAccess = (1<<31) | (1<<30) | (0<<29) | (RegAddr<<21) | (PhyAddr<<16) | RegValue;
		while(HWREG(pruss_mdio_addr)&0x80000000)
		{
			;
		}
		HWREG(pruss_mdio_addr) = UserAccess;
	}else
	{
		return(-1);
	}
	return(0);
}
#endif //FASTBOOT_PHY
/******************************************************************************
**                              END OF FILE
*******************************************************************************/
